# -*- coding: utf-8 -*-

from abc import abstractmethod
from typing import TypeVar, Tuple, List, Dict, Callable, Optional, Generic, Any
from numpy import ndarray

FVal = TypeVar('FVal')


class Objective(Generic[FVal]):

    """Objective function base class."""

    @abstractmethod
    def get_upper(self) -> ndarray:
        """Return upper bound."""
        ...

    @abstractmethod
    def get_lower(self) -> ndarray:
        """Return lower bound."""
        ...

    @abstractmethod
    def result(self, v: ndarray) -> FVal:
        """Show the result."""
        ...


class AlgorithmBase(Generic[FVal]):

    """Algorithm base class."""

    func: Objective[FVal]

    def __class_getitem__(cls, item):
        # PEP 560
        raise NotImplemented

    def __init__(
        self,
        func: Objective[FVal],
        settings: Dict[str, Any],
        progress_fun: Optional[Callable[[int, str], None]] = None,
        interrupt_fun: Optional[Callable[[], bool]] = None
    ):
        ...

    def history(self) -> List[Tuple[int, float, float]]:
        """Return the history of the process."""
        ...

    def run(self) -> FVal:
        """Init and run GA for max_gen times."""
        ...



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Triangular expressions.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from typing import Sequence, Iterator
from libc.math cimport sin, cos, M_PI
from .expression cimport VJoint, VPoint, VLink


cdef inline str symbol_str(symbol p):
    """Pair to string."""
    if p.first == P_LABEL:
        return f"P{p.second}"
    elif p.first == L_LABEL:
        return f"L{p.second}"
    elif p.first == A_LABEL:
        return f"a{p.second}"
    elif p.first == S_LABEL:
        return f"S{p.second}"
    else:
        return ""


cdef class ExpressionStack:

    """The stack of Python wrapper."""

    cdef void add_pla(self, symbol c1, symbol v1, symbol v2, symbol target):
        cdef Expression e
        e.func = PLA
        e.c1 = c1
        e.v1 = v1
        e.v2 = v2
        e.c2 = target
        e.op = False
        self.stack.push_back(e)

    cdef void add_plap(self, symbol c1, symbol v1, symbol v2, symbol c2, symbol target):
        cdef Expression e
        e.func = PLAP
        e.c1 = c1
        e.v1 = v1
        e.v2 = v2
        e.c2 = c2
        e.c3 = target
        e.op = False
        self.stack.push_back(e)

    cdef void add_pllp(self, symbol c1, symbol v1, symbol v2, symbol c2, symbol target):
        cdef Expression e
        e.func = PLLP
        e.c1 = c1
        e.v1 = v1
        e.v2 = v2
        e.c2 = c2
        e.c3 = target
        e.op = False
        self.stack.push_back(e)

    cdef void add_plpp(self, symbol c1, symbol v1, symbol c2, symbol c3, symbol target, bint op):
        cdef Expression e
        e.func = PLPP
        e.c1 = c1
        e.v1 = v1
        e.c2 = c2
        e.c3 = c3
        e.c4 = target
        e.op = op
        self.stack.push_back(e)

    cdef void add_pxy(self, symbol c1, symbol v1, symbol v2, symbol target):
        cdef Expression e
        e.func = PXY
        e.c1 = c1
        e.v1 = v1
        e.v2 = v2
        e.c2 = target
        e.op = False
        self.stack.push_back(e)

    cpdef list as_list(self):
        """Turn into normal list."""
        stack = []
        cdef Expression expr
        for expr in self.stack:
            if expr.func == PLA:
                stack.append((
                    "PLAP",
                    symbol_str(expr.c1),
                    symbol_str(expr.v1),
                    symbol_str(expr.v2),
                    symbol_str(expr.c2),
                ))
            elif expr.func == PLAP:
                stack.append((
                    "PLAP",
                    symbol_str(expr.c1),
                    symbol_str(expr.v1),
                    symbol_str(expr.v2),
                    symbol_str(expr.c2),
                    symbol_str(expr.c3),
                ))
            elif expr.func == PLLP:
                stack.append((
                    "PLLP",
                    symbol_str(expr.c1),
                    symbol_str(expr.v1),
                    symbol_str(expr.v2),
                    symbol_str(expr.c2),
                    symbol_str(expr.c3),
                ))
            elif expr.func == PLPP:
                stack.append((
                    "PLPP",
                    symbol_str(expr.c1),
                    symbol_str(expr.v1),
                    symbol_str(expr.c2),
                    symbol_str(expr.c3),
                    symbol_str(expr.c4),
                ))
            elif expr.func == PXY:
                stack.append((
                    "PXY",
                    symbol_str(expr.c1),
                    symbol_str(expr.v1),
                    symbol_str(expr.v2),
                    symbol_str(expr.c2),
                ))
        return stack

    def __repr__(self) -> str:
        return f"{type(self).__name__}({self.as_list()})"


cdef inline bint _is_all_lock(dict status):
    """Test is all status done."""
    cdef int node
    cdef bint n_status
    for node, n_status in status.items():
        if not n_status:
            return False
    return True


cdef inline bint _clockwise(tuple c1, tuple c2, tuple c3):
    """Check orientation of three points."""
    cdef double val = (c2[1] - c1[1]) * (c3[0] - c2[0]) - (c2[0] - c1[0]) * (c3[1] - c2[1])
    return val == 0 or val > 0


def _get_reliable_friend(
    node: int,
    vpoints: Sequence[VPoint],
    vlinks: dict,
    status: dict
) -> Iterator[int]:
    """Return a generator yield the vertices
        that "has been solved" on the same link.
    """
    cdef int friend
    for link in vpoints[node].links:
        if len(vlinks[link]) < 2:
            continue
        for friend in vlinks[link]:
            if status[friend] and friend != node:
                yield friend


def _get_not_base_friend(
    node: int,
    vpoints: Sequence[VPoint],
    vlinks: dict,
    status: dict
) -> Iterator[int]:
    """Get a friend from constrained nodes."""
    if len(vpoints[node].links) < 2:
        raise StopIteration
    cdef int friend
    for friend in vlinks[vpoints[node].links[1]]:
        if status[friend]:
            yield friend


def _get_base_friend(
    node: int,
    vpoints: Sequence[VPoint],
    vlinks: dict,
    status: dict
) -> Iterator[int]:
    """Get the constrained node of same links."""
    if len(vpoints[node].links) < 1:
        raise StopIteration
    cdef int friend
    for friend in vlinks[vpoints[node].links[0]]:
        yield friend


cdef inline int _get_input_base(int node, object inputs):
    """Get the base node for input pairs."""
    cdef int base, node_
    for base, node_ in inputs:
        if node == node_:
            return base
    return -1


cpdef ExpressionStack vpoints_configure(
    object vpoints_,
    object inputs,
    dict status = None
):
    """Auto configuration algorithm.
    
    For VPoint list.
    + vpoints_: [vpoint0, vpoint1, ...]
    + inputs: [(p0, p1), (p0, p2), ...]
    + status: Dict[int, bint]
    
    vpoints will make a copy that we don't want to modified itself.
    """
    if inputs is None:
        inputs = ()
    if status is None:
        status = {}

    if not vpoints_:
        return ExpressionStack.__new__(ExpressionStack)
    if not inputs:
        return ExpressionStack.__new__(ExpressionStack)

    vpoints = list(vpoints_)
    cdef int vpoints_count = len(vpoints)

    # First, we create a "VLinks" that can help us to
    # find a relationship just like adjacency matrix.
    cdef int node
    cdef VPoint vpoint
    vlinks = {}
    for node, vpoint in enumerate(vpoints):
        status[node] = False
        if vpoint.links:
            for link in vpoint.links:
                # Connect on the ground and it is not a slider.
                if link == VLink.FRAME and vpoint.type == VJoint.R:
                    status[node] = True
                # Add as vlink.
                if link not in vlinks:
                    vlinks[link] = {node}
                else:
                    vlinks[link].add(node)
        else:
            status[node] = True

    # Replace the P joints and their friends with RP joint.
    # DOF must be same after properties changed.
    cdef int base
    cdef VPoint vpoint_
    for base in range(vpoints_count):
        vpoint = vpoints[base]
        if vpoint.type != VJoint.P or not vpoint.grounded():
            continue
        for link in vpoint.links[1:]:
            links = set()
            for node in vlinks[link]:
                vpoint_ = vpoints[node]
                if node == base or vpoint_.type != VJoint.R:
                    continue
                links.update(vpoint_.links)
                vpoints[node] = VPoint.c_slider_joint(
                    [vpoint.links[0]] + [
                        link_ for link_ in vpoint_.links
                        if (link_ not in vpoint.links)
                    ],
                    VJoint.RP,
                    vpoint.angle,
                    vpoint_.x,
                    vpoint_.y
                )

    # Add positions parameters.
    pos = []
    for vpoint in vpoints:
        pos.append(vpoint.c[0 if vpoint.type == VJoint.R else 1])

    cdef ExpressionStack exprs = ExpressionStack.__new__(ExpressionStack)
    cdef int link_symbol = 0
    cdef int angle_symbol = 0

    # Input joints (R) that was connect with ground.
    for base, node in inputs:
        if status[base]:
            exprs.add_pla(
                [P_LABEL, base],
                [L_LABEL, link_symbol],
                [A_LABEL, angle_symbol],
                [P_LABEL, node]
            )
            status[node] = True
            link_symbol += 1
            angle_symbol += 1

    # Now let we search around all of points, until find the solutions that we could.
    input_targets = {node for base, node in inputs}
    node = 0
    cdef int skip_times = 0
    cdef int around = len(status)

    cdef int friend_a, friend_b, friend_c, friend_d
    cdef double tmp_x, tmp_y, angle
    # Friend iterator
    while not _is_all_lock(status):
        if node not in status:
            node = 0
            continue
        # Check and break the loop if it's re-scan again.
        if skip_times >= around:
            break
        if status[node]:
            node += 1
            skip_times += 1
            continue
        vpoint = vpoints[node]
        if vpoint.type == VJoint.R:
            # R joint
            # + Is input node?
            # + Normal revolute joint.
            if node in input_targets:
                base = _get_input_base(node, inputs)
                if status[base]:
                    exprs.add_pla(
                        [P_LABEL, base],
                        [L_LABEL, link_symbol],
                        [A_LABEL, angle_symbol],
                        [P_LABEL, node]
                    )
                    status[node] = True
                    link_symbol += 1
                    angle_symbol += 1
                else:
                    skip_times += 1
            else:
                fi = _get_reliable_friend(node, vpoints, vlinks, status)
                try:
                    friend_a = next(fi)
                    friend_b = next(fi)
                except StopIteration:
                    skip_times += 1
                else:
                    if not _clockwise(pos[friend_a], pos[node], pos[friend_b]):
                        friend_a, friend_b = friend_b, friend_a
                    exprs.add_pllp(
                        [P_LABEL, friend_a],
                        [L_LABEL, link_symbol],
                        [L_LABEL, link_symbol + 1],
                        [P_LABEL, friend_b],
                        [P_LABEL, node]
                    )
                    status[node] = True
                    link_symbol += 2
                    skip_times = 0

        elif vpoint.type == VJoint.P:
            # Need to solve P joint itself here (only grounded)
            fi = _get_not_base_friend(node, vpoints, vlinks, status)
            try:
                if vpoints[node].pin_grounded():
                    raise StopIteration
                if not vpoints[node].grounded():
                    raise StopIteration
                if vpoints[node].has_offset():
                    raise StopIteration
                friend_a = next(fi)
            except StopIteration:
                skip_times += 1
            else:
                exprs.add_pxy(
                    [P_LABEL, friend_a],
                    [L_LABEL, link_symbol],
                    [L_LABEL, link_symbol + 1],
                    [P_LABEL, node]
                )
                status[node] = True
                link_symbol += 2
                # Solution for all friends.
                for link in vpoints[node].links[1:]:
                    for friend_b in vlinks[link]:
                        if status[friend_b]:
                            continue
                        exprs.add_pxy(
                            [P_LABEL, node],
                            [L_LABEL, link_symbol],
                            [L_LABEL, link_symbol + 1],
                            [P_LABEL, friend_b]
                        )
                        status[friend_b] = True
                        link_symbol += 2
                skip_times = 0

        elif vpoint.type == VJoint.RP:
            # RP joint
            fi = _get_base_friend(node, vpoints, vlinks, status)
            # Copy as 'friend_c'.
            friend_c = node
            # 'S' point.
            tmp_x, tmp_y = pos[node]
            angle = vpoints[node].angle / 180 * M_PI
            tmp_x += cos(angle)
            tmp_y += sin(angle)
            try:
                if vpoints[node].pin_grounded():
                    raise StopIteration
                if not vpoints[node].grounded():
                    raise StopIteration
                if vpoints[node].has_offset():
                    raise StopIteration
                friend_a = next(_get_not_base_friend(node, vpoints, vlinks, status))
                friend_b = next(fi)
                # Slot is not grounded.
                if not vpoints[node].grounded():
                    friend_d = next(fi)
                    if not _clockwise(pos[friend_b], (tmp_x, tmp_y), pos[friend_d]):
                        friend_b, friend_d = friend_d, friend_b
                    exprs.add_pllp(
                        [P_LABEL, friend_b],
                        [L_LABEL, link_symbol],
                        [L_LABEL, link_symbol + 1],
                        [P_LABEL, friend_d],
                        [P_LABEL, node]
                    )
                    link_symbol += 2
            except StopIteration:
                skip_times += 1
            else:
                # PLPP
                # [PLLP]
                # Set 'S' (slider) point to define second point of slider.
                # + A 'friend' from base link.
                # + Get distance from me and friend.
                # [PLPP]
                # Re-define coordinate of target point by self and 'S' point.
                # + A 'friend' from other link.
                # + Solve.
                if not _clockwise(pos[friend_b], (tmp_x, tmp_y), pos[friend_c]):
                    friend_b, friend_c = friend_c, friend_b
                exprs.add_pllp(
                    [P_LABEL, friend_b],
                    [L_LABEL, link_symbol],
                    [L_LABEL, link_symbol + 1],
                    [P_LABEL, friend_c],
                    [S_LABEL, node]
                )
                # Two conditions.
                exprs.add_plpp(
                    [P_LABEL, friend_a],
                    [L_LABEL, link_symbol + 2],
                    [P_LABEL, node],
                    [S_LABEL, node],
                    [P_LABEL, node],
                    (pos[friend_a][0] - pos[node][0] > 0) != (vpoints[node].angle > 90)
                )
                status[node] = True
                link_symbol += 3
                skip_times = 0

        node += 1

    # exprs: [('PLAP', 'P0', 'L0', 'a0', 'P1', 'P2'), ...]
    return exprs



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Number synthesis.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from numpy import (
    int16 as np_int,
    array as np_array,
    zeros as np_zeros,
    sum as np_sum,
    prod as np_prod,
    repeat as np_repeat,
    arange,
)


@cython.boundscheck(False)
cdef int16_t[:, :] product(tuple pool, object stop_func):
    """Product function as same as iteration tools.

    The pool is created by range(n).
    """
    if not pool:
        return np_array([], dtype=np_int)

    cdef int16_t[:] tmp1
    cdef int16_t[:, :] tmp2
    cdef int16_t[:] array0 = arange(pool[0], dtype=np_int)

    cdef int n = np_prod(pool)
    cdef int16_t[:, :] out = np_zeros((n, len(pool)), dtype=np_int)

    cdef int array0_size = len(array0)
    cdef int m = n / array0_size
    tmp1 = np_repeat(array0, m)
    out[:, 0] = tmp1

    cdef int j
    if pool[1:]:
        tmp2 = product(pool[1:], stop_func)
        out[0:m, 1:] = tmp2
        for j in range(1, array0_size):
            if stop_func is not None and stop_func():
                return out
            out[j * m:(j + 1) * m, 1:] = out[0:m, 1:]
    return out


cdef int16_t[:, :] _product(int pool_size, int repeat, object stop_func):
    return product((pool_size,) * repeat + (1,), stop_func)


cdef inline int _m_max(int nl, int nj) nogil:
    """Find max number of joint on each link.

    + nl <= nj and nj <= (2 * nl - 3)
    + (2 * nl - 3) <= nj and nj <= (nl * (nl - 1) / 2)
    + other exceptions (return -1).
    """
    if nl <= nj <= (2 * nl - 3):
        return nj - nl + 2
    if nl == nj == 0:
        return -1
    if 2 * nl - 3 <= nj <= <double>(nl * (nl - 1)) / 2:
        return nl - 1
    return -1


cdef inline int _sum_factors(int16_t[:] factors):
    """F0*N2 + F1*N3 + F2*N4 + ... + Fn*N(n+2)"""
    cdef int factor = 0
    cdef int i
    for i in range(len(factors)):
        factor += factors[i] * (i + 2)
    return factor


cpdef list link_synthesis(int nl, int nj, object stop_func = None):
    """Number synthesis try and error function."""
    result = []
    cdef int m_max_v = _m_max(nl, nj)
    if m_max_v == -1:
        raise ValueError("incorrect mechanism.")

    cdef int i, p
    cdef int16_t[:] symbols
    for symbols in _product(nl + 1, m_max_v - 2, stop_func):
        nl_m_max = nl - np_sum(symbols)
        if nl_m_max < 0:
            continue
        symbols[-1] = nl_m_max
        if _sum_factors(symbols) == nj * 2:
            result.append(tuple(symbols))
    return result


cdef inline int _j_m(int16_t[:] link_num):
    """Return value of JM."""
    cdef int num
    cdef int i = 3
    cdef float c = 0
    for num in link_num[1:]:
        c += <double>i / 2 * num
        i += 1
    return <int>c


cdef inline int _j_m_p(int n_m) nogil:
    """Return value of J'M. This is improved function.

    + Origin equation:
    if n_m % 2 == 0:
        return <int>((3 * (n_m - 1) - 1) / 2)
    else:
        return <int>((3 * (n_m - 1) - 2) / 2)
    """
    # Number of multiple links.
    if n_m <= 1:
        return 0
    elif n_m == 2:
        return 1
    else:
        return 3 * (n_m - 2)


cpdef list contracted_link_synthesis(object link_num_list, object stop_func = None):
    """Generate the contracted link assortment."""
    cdef int16_t[:] link_num
    if len(link_num_list) == 1:
        link_num = np_zeros(link_num_list[0], dtype=np_int)
        link_num[-1] = 1
        return [tuple(link_num)]

    link_num = np_array(link_num_list, dtype=np_int)

    # Contracted link
    cdef int j_m_v = _j_m(link_num)
    cdef int n_c_min = max(1, j_m_v - _j_m_p(sum(link_num[1:])))
    cdef int n_c_max = min(link_num[0], j_m_v)

    # i = NL2 - NC + 2
    cdef int i_max = min(link_num[0], link_num[0] - n_c_min + 2)

    # Matching formula
    cdef int count, factor, index
    cdef float last_factor
    cdef int16_t[:] m
    cj_list = []
    for m in _product(link_num[0] + 1, i_max - 1, stop_func):
        count = 0
        index = 1
        for factor in m[:-1]:
            count +=  factor * index
            index += 1

        # Check if the last factor is a natural number.
        last_factor = <double>(link_num[0] - count) / index
        factor = <int>last_factor
        if last_factor < 0 or last_factor != factor:
            continue

        m[-1] = factor
        if n_c_min <= np_sum(m) <= n_c_max:
            cj_list.append(tuple(m))

    return cj_list



# -*- coding: utf-8 -*-

"""Lark parser to parse the expression."""

__author__ = "Yuan Chang"
__copyright__ = "Copyright (C) 2016-2019"
__license__ = "AGPL"
__email__ = "pyslvs@gmail.com"

from abc import abstractmethod
from typing import (
    cast,
    Tuple,
    List,
    Dict,
    Iterator,
    Optional,
    Union,
    TypeVar,
    Generic,
)
from dataclasses import dataclass
from lark import Lark, Transformer, LexError
from .expression import get_vlinks, VJoint, VPoint, VLink
from .graph import Graph

_T1 = TypeVar('_T1')
_T2 = TypeVar('_T2')
_Coord = Tuple[float, float]
_JointArgs = List[Union[str, VJoint, float, _Coord, Tuple[str, ...]]]

# Color dictionary
_color_list: Dict[str, Tuple[int, int, int]] = {
    'Red': (172, 68, 68),
    'Green': (110, 190, 30),
    'Blue': (68, 120, 172),
    'Cyan': (0, 255, 255),
    'Magenta': (255, 0, 255),
    'Brick-Red': (255, 130, 130),
    'Yellow': (255, 255, 0),
    'Gray': (160, 160, 160),
    'Orange': (225, 165, 0),
    'Pink': (225, 192, 230),
    'Black': (0, 0, 0),
    'White': (255, 255, 255),
    'Dark-Red': (128, 0, 0),
    'Dark-Green': (0, 128, 0),
    'Dark-Blue': (0, 0, 128),
    'Dark-Cyan': (128, 0, 128),
    'Dark-Magenta': (255, 0, 255),
    'Dark-Yellow': (128, 128, 0),
    'Dark-Gray': (128, 128, 128),
    'Dark-Orange': (225, 140, 0),
    'Dark-Pink': (225, 20, 147),
}
color_names = tuple(sorted(_color_list.keys()))


def color_rgb(name: str) -> Tuple[int, int, int]:
    """Get color by name.

    Usage:
    >>> color_rgb("Black")
    (0, 0, 0)
    >>> color_rgb("(255, 255, 255)")
    (255, 255, 255)
    """
    if name in _color_list:
        return _color_list[name]
    else:
        try:
            # Input RGB as a "(255, 255, 255)" string
            rgb = (
                name.replace('(', '')
                .replace(')', '')
                .replace(" ", '')
                .split(',', maxsplit=3)
            )
            color_text = (int(rgb[0]), int(rgb[1]), int(rgb[2]))
        except ValueError:
            return 0, 0, 0
        else:
            return color_text


@dataclass(repr=False, eq=False)
class PointArgs:

    """Point table argument."""

    links: str
    type: str
    color: str
    x: float
    y: float


@dataclass(repr=False, eq=False)
class LinkArgs:

    """Link table argument."""

    name: str
    color: str
    points: str


_GRAMMAR = Lark(r"""
    // Number
    DIGIT: "0".."9"
    INT: DIGIT+
    SIGNED_INT: ["+" | "-"] INT
    DECIMAL: INT "." INT? | "." INT
    _EXP: ("e" | "E") SIGNED_INT
    FLOAT: INT _EXP | DECIMAL _EXP?
    NUMBER: FLOAT | INT
    SIGNED_NUMBER: ["+" | "-"] NUMBER

    // Letters
    LCASE_LETTER: "a".."z"
    UCASE_LETTER: "A".."Z"
    LETTER: UCASE_LETTER | LCASE_LETTER | "_"
    CNAME: LETTER (LETTER | DIGIT)*

    // White space and new line
    WS: /\s+/
    CR: /\r/
    LF: /\n/
    NEWLINE: (CR? LF)+
    %ignore WS
    %ignore NEWLINE

    // Comment
    LINE_COMMENT: /#[^\n]*/
    MULTILINE_COMMENT: /#\[[\s\S]*#\][^\n]*/
    %ignore LINE_COMMENT
    %ignore MULTILINE_COMMENT

    // Custom data type
    JOINT_TYPE: "RP" | "R" | "P"
    COLOR: """ + "|".join(f'"{color}"' for color in color_names) + r"""
    type: JOINT_TYPE
    name: CNAME
    number: SIGNED_NUMBER
    color_value: INT

    // Main grammar
    joint: "J[" type ["," angle] ["," color] "," point "," link "]"
    link: "L[" [name ("," name)* ","?] "]"
    point: "P[" number "," number "]"
    angle: "A[" number "]"
    color: "color[" (("(" color_value ("," color_value) ~ 2 ")") | COLOR) "]"
    mechanism: "M[" [joint ("," joint)* ","?] "]"
    ?start: mechanism
""", parser='lalr')


class _Transformer(Transformer, Generic[_T1, _T2]):

    """Base transformer implementation."""

    @staticmethod
    @abstractmethod
    def type(n: List[str]) -> _T1:
        ...

    @staticmethod
    def name(n: List[str]) -> str:
        return str(n[0])

    @staticmethod
    def color(n: List[str]) -> str:
        return str(n[0]) if len(n) == 1 else str(tuple(n))

    @staticmethod
    def color_value(n: List[str]) -> int:
        return int(n[0])

    @staticmethod
    def number(n: List[str]) -> float:
        return float(n[0])

    angle = number

    @staticmethod
    def point(c: List[float]) -> _Coord:
        return c[0], c[1]

    @staticmethod
    def link(a: List[str]) -> Tuple[str, ...]:
        return tuple(a)

    @staticmethod
    @abstractmethod
    def joint(args: _JointArgs) -> _T2:
        ...

    @staticmethod
    def mechanism(joints: List[_T2]) -> List[_T2]:
        return joints


class _ParamsTrans(_Transformer[str, PointArgs]):

    """Transformer will parse into a list of VPoint data."""

    @staticmethod
    def type(n: List[str]) -> str:
        return str(n[0])

    @staticmethod
    def joint(args: _JointArgs) -> PointArgs:
        """Sort the argument list.

        [0]: type
        ([1]: angle)
        ([2]: color)
        [-2]: point (coordinate)
        [-1]: link
        """
        type_str = cast(str, args[0])
        x, y = cast(_Coord, args[-2])
        links = ','.join(cast(Tuple[str, ...], args[-1]))
        if type_str == 'R':
            if len(args) == 3:
                return PointArgs(links, 'R', 'Green', x, y)
            elif len(args) == 4:
                color = cast(str, args[-3])
                return PointArgs(links, 'R', color, x, y)
        else:
            angle = cast(float, args[1])
            type_angle = f'{type_str}:{angle}'
            if len(args) == 4:
                return PointArgs(links, type_angle, 'Green', x, y)
            elif len(args) == 5:
                color = cast(str, args[-3])
                return PointArgs(links, type_angle, color, x, y)

        raise LexError(f"invalid options: {args}")


class _PositionTrans(_Transformer[str, _Coord]):

    """Transformer will parse into a list of position data."""

    @staticmethod
    def type(n: List[str]) -> str:
        return str(n[0])

    @staticmethod
    def joint(args: _JointArgs) -> _Coord:
        x, y = cast(_Coord, args[-2])
        return x, y


class _VPointsTrans(_Transformer[VJoint, VPoint]):

    """Using same grammar return as VPoints."""

    @staticmethod
    def type(n: List[str]) -> VJoint:
        """Return as int type."""
        type_str = str(n[0])
        if type_str == 'R':
            return VJoint.R
        elif type_str == 'P':
            return VJoint.P
        elif type_str == 'RP':
            return VJoint.RP
        else:
            raise ValueError(f"invalid joint type: {type_str}")

    @staticmethod
    def joint(args: _JointArgs) -> VPoint:
        """Sort the argument list.

        [0]: type
        ([1]: angle)
        ([2]: color)
        [-2]: point (coordinate)
        [-1]: link
        """
        type_int = cast(VJoint, args[0])
        x, y = cast(_Coord, args[-2])
        links = cast(Tuple[str, ...], args[-1])
        if type_int == VJoint.R:
            if len(args) == 3:
                return VPoint.r_joint(links, x, y)
            elif len(args) == 4:
                color = cast(str, args[-3])
                return VPoint(links, VJoint.R, 0., color, x, y, color_rgb)
        else:
            angle = cast(float, args[1])
            if len(args) == 4:
                return VPoint.slider_joint(links, type_int, angle, x, y)
            elif len(args) == 5:
                color = cast(str, args[-3])
                return VPoint(links, type_int, angle, color, x, y, color_rgb)

        raise LexError(f"invalid options: {args}")


_params_translator = _ParamsTrans()
_pos_translator = _PositionTrans()
_vpoint_translator = _VPointsTrans()


def parse_params(expr: str) -> List[PointArgs]:
    """Using to parse the expression and return arguments."""
    return _params_translator.transform(_GRAMMAR.parse(expr))


def parse_pos(expr: str) -> List[_Coord]:
    """Using to parse the expression and return arguments."""
    return _pos_translator.transform(_GRAMMAR.parse(expr))


def parse_vpoints(expr: str) -> List[VPoint]:
    """Parse as VPoints."""
    return _vpoint_translator.transform(_GRAMMAR.parse(expr))


def parse_vlinks(expr: str) -> List[VLink]:
    """Parse as VLinks."""
    return get_vlinks(parse_vpoints(expr))


def _sorted_pair(a: int, b: int) -> Tuple[int, int]:
    return (a, b) if a < b else (b, a)


def edges_view(graph: Graph) -> Iterator[Tuple[int, Tuple[int, int]]]:
    """This generator can keep the numbering be consistent."""
    yield from enumerate(sorted(_sorted_pair(n1, n2) for n1, n2 in graph.edges))


def graph2vpoints(
    graph: Graph,
    pos: Dict[int, _Coord],
    cus: Optional[Dict[int, int]] = None,
    same: Optional[Dict[int, int]] = None,
    grounded: Optional[int] = None
) -> List[VPoint]:
    """Change NetworkX graph into VPoints.

    cus: custom vertices (not joint)
        {node_name: link_number}
    same: multiple joints
        {n1: n2, n3: n2} => (n1 as n2) and (n3 as n2)
    """
    if cus is None:
        cus = {}
    if same is None:
        same = {}

    same_r: Dict[int, List[int]] = {}
    for k, v in same.items():
        if v in same_r:
            same_r[v].append(k)
        else:
            same_r[v] = [k]
    tmp_list = []
    ev = dict(edges_view(graph))
    for i, edge in ev.items():
        if i in same:
            # Do not connect to anyone!
            continue
        edges = set(edge)
        if i in same_r:
            for j in same_r[i]:
                edges.update(set(ev[j]))
        x, y = pos[i]
        links = [
            f"L{link}" if link != grounded else VLink.FRAME for link in edges
        ]
        tmp_list.append(VPoint.r_joint(links, x, y))
    for name in sorted(cus):
        link = f"L{cus[name]}" if cus[name] != grounded else VLink.FRAME
        x, y = pos[name]
        tmp_list.append(VPoint.r_joint((link,), x, y))
    return tmp_list



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Planarity check function for Graph class.

The algorithms references:
+ NetworkX

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from collections import defaultdict
from libcpp.pair cimport pair as cpair
from libcpp.list cimport list as clist
from libcpp.map cimport map as cmap

ctypedef cpair[int, int] ipair


cpdef inline bint is_planar(Graph g):
    """Return True if the graph is planar.
    
    Only this function well be shown as public.
    """
    cdef _LRPlanarity planarity_state = _LRPlanarity.__new__(_LRPlanarity, g)
    return planarity_state.lr_planarity() is not None


cdef inline _ConflictPair _top_of_stack(list l):
    """Returns the conflict pair element on top of the stack."""
    if not l:
        return None
    return l[-1]


cdef class _LRPlanarity:

    """A class to maintain the state during planarity check."""

    cdef Graph g, DG
    cdef _PlanarEmbedding embedding
    cdef list S
    cdef dict lowpt, lowpt2, nesting_depth, adjs
    cdef dict ordered_adjs, stack_bottom, lowpt_edge, left_ref, right_ref

    # default dict
    cdef object height, parent_edge, ref, side

    def __cinit__(self, Graph g):
        # copy G without adding self-loops
        self.g = Graph.__new__(Graph, set(g.edges))

        # distance from tree root
        self.height = defaultdict(lambda: None)

        self.lowpt = {}  # height of lowest return point of an edge
        self.lowpt2 = {}  # height of second lowest return point
        self.nesting_depth = {}  # for nesting order

        # None -> missing edge
        self.parent_edge = defaultdict(lambda: None)

        # oriented DFS graph
        self.DG = Graph.__new__(Graph, [])
        self.DG.add_vertices(g.vertices)

        self.adjs = {}
        self.ordered_adjs = {}

        self.ref = defaultdict(lambda: None)
        self.side = defaultdict(lambda: 1)

        # stack of conflict pairs
        self.S = []
        self.stack_bottom = {}
        self.lowpt_edge = {}

        self.left_ref = {}
        self.right_ref = {}

        self.embedding = _PlanarEmbedding.__new__(_PlanarEmbedding, [])

    cdef _PlanarEmbedding lr_planarity(self):
        """Execute the LR planarity test."""
        if len(self.g.vertices) > 2 and len(self.g.edges) > 3 * len(self.g.vertices) - 6:
            # graph is not planar
            return None

        # make adjacency lists for dfs
        cdef int v
        for v in self.g.vertices:
            self.adjs[v] = list(self.g.adj[v])

        # orientation of the graph by depth first search traversal
        cdef clist[int] roots
        for v in self.g.vertices:
            if self.height[v] is None:
                self.height[v] = 0
                roots.push_back(v)
                self.dfs_orientation(v)

        # Free no longer used variables
        self.g = None
        self.lowpt2 = None
        self.adjs = None

        # testing
        for v in self.DG.vertices:  # sort the adjacency lists by nesting depth
            # note: this sorting leads to non linear time
            self.ordered_adjs[v] = sorted(
                [n2 for n1, n2 in self.DG.edges if n1 == v],
                key=lambda x: self.nesting_depth[v, x]
            )
        for v in roots:
            if not self.dfs_testing(v):
                return None

        # Free no longer used variables
        self.height = None
        self.lowpt = None
        self.S = None
        self.stack_bottom = None
        self.lowpt_edge = None
        for e in self.DG.edges:
            self.nesting_depth[e] = self.sign(e) * self.nesting_depth[e]
        self.embedding.add_vertices(self.DG.vertices)
        cdef int previous_node, w
        for v in self.DG.vertices:
            # sort the adjacency lists again
            self.ordered_adjs[v] = sorted(
                [n2 for n1, n2 in self.DG.edges if n1 == v],
                key=lambda x: self.nesting_depth[v, x]
            )
            # initialize the embedding
            previous_node = -1
            for w in self.ordered_adjs[v]:
                self.embedding.add_half_edge_cw(v, w, previous_node)
                previous_node = w

        # Free no longer used variables
        self.DG = None
        self.nesting_depth = None
        self.ref = None

        # compute the complete embedding
        for v in roots:
            self.dfs_embedding(v)

        return self.embedding

    cdef bint dfs_testing(self, int v):
        """Test for LR partition."""
        # the recursion stack
        cdef clist[int] dfs_stack = [v]
        # index of next edge to handle in adjacency list of each node
        cdef cmap[int, int] ind
        # bintean to indicate whether to skip the initial work for an edge
        skip_init = defaultdict(lambda: False)
        cdef bint skip_final
        cdef int w
        while not dfs_stack.empty():
            v = dfs_stack.back()
            dfs_stack.pop_back()
            e = self.parent_edge[v]
            # to indicate whether to skip the final block after the for loop
            skip_final = False

            for w in self.ordered_adjs[v][ind[v]:]:
                ei = (v, w)
                if not skip_init[ei]:
                    self.stack_bottom[ei] = _top_of_stack(self.S)
                    if ei == self.parent_edge[w]:  # tree edge
                        dfs_stack.push_back(v)  # revisit v after finishing w
                        dfs_stack.push_back(w)  # visit w next
                        skip_init[ei] = True  # don't redo this block
                        skip_final = True  # skip final work after breaking
                        break  # handle next node in dfs_stack (i.e. w)
                    else:  # back edge
                        self.lowpt_edge[ei] = ei
                        self.S.append(_ConflictPair.__new__(_ConflictPair, right=_Interval(ei, ei)))

                # integrate new return edges
                if self.lowpt[ei] < self.height[v]:
                    if w == self.ordered_adjs[v][0]:  # e_i has return edge
                        self.lowpt_edge[e] = self.lowpt_edge[ei]
                    else:  # add constraints of e_i
                        if not self.add_constraints(ei, e):
                            # graph is not planar
                            return False

                ind[v] += 1

            if not skip_final:
                # remove back edges returning to parent
                if e is not None:  # v isn't root
                    self.remove_back_edges(e)

        return True

    cdef void dfs_embedding(self, int v):
        """Completes the embedding."""
        # the recursion stack
        cdef clist[int] dfs_stack = [v]
        # index of next edge to handle in adjacency list of each node
        cdef cmap[int, int] ind

        cdef int w
        while not dfs_stack.empty():
            v = dfs_stack.back()
            dfs_stack.pop_back()

            for w in self.ordered_adjs[v][ind[v]:]:
                ind[v] += 1
                ei = (v, w)

                if ei == self.parent_edge[w]:  # tree edge
                    self.embedding.add_half_edge_first(w, v)
                    self.left_ref[v] = w
                    self.right_ref[v] = w

                    dfs_stack.push_back(v)  # revisit v after finishing w
                    dfs_stack.push_back(w)  # visit w next
                    break  # handle next node in dfs_stack (i.e. w)
                else:  # back edge
                    if self.side[ei] == 1:
                        self.embedding.add_half_edge_cw(w, v, self.right_ref[w])
                    else:
                        self.embedding.add_half_edge_ccw(w, v, self.left_ref[w])
                        self.left_ref[w] = v

    cdef void dfs_orientation(self, int v):
        """Orient the graph by DFS, compute lowpoints and nesting order."""
        # the recursion stack
        cdef clist[int] dfs_stack = [v]
        # index of next edge to handle in adjacency list of each node
        cdef cmap[int, int] ind
        # bintean to indicate whether to skip the initial work for an edge
        skip_init = defaultdict(lambda: False)
        cdef int w
        while not dfs_stack.empty():
            v = dfs_stack.back()
            dfs_stack.pop_back()
            e = self.parent_edge[v]
            for w in self.adjs[v][ind[v]:]:
                vw = (v, w)
                if not skip_init[vw]:
                    if (v, w) in self.DG.edges or (w, v) in self.DG.edges:
                        ind[v] += 1
                        continue  # the edge was already oriented
                    self.DG.add_edge(v, w)  # orient the edge
                    self.lowpt[vw] = self.height[v]
                    self.lowpt2[vw] = self.height[v]
                    if self.height[w] is None:  # (v, w) is a tree edge
                        self.parent_edge[w] = vw
                        self.height[w] = self.height[v] + 1
                        dfs_stack.push_back(v)  # revisit v after finishing w
                        dfs_stack.push_back(w)  # visit w next
                        skip_init[vw] = True  # don't redo this block
                        break  # handle next node in dfs_stack (i.e. w)
                    else:  # (v, w) is a back edge
                        self.lowpt[vw] = self.height[w]
                # determine nesting graph
                self.nesting_depth[vw] = 2 * self.lowpt[vw]
                if self.lowpt2[vw] < self.height[v]:  # chordal
                    self.nesting_depth[vw] += 1
                # update lowpoints of parent edge e
                if e is not None:
                    if self.lowpt[vw] < self.lowpt[e]:
                        self.lowpt2[e] = min(self.lowpt[e], self.lowpt2[vw])
                        self.lowpt[e] = self.lowpt[vw]
                    elif self.lowpt[vw] > self.lowpt[e]:
                        self.lowpt2[e] = min(self.lowpt2[e], self.lowpt[vw])
                    else:
                        self.lowpt2[e] = min(self.lowpt2[e], self.lowpt2[vw])
                ind[v] += 1

    cdef bint add_constraints(self, tuple ei, tuple e):
        cdef _ConflictPair P = _ConflictPair.__new__(_ConflictPair)

        # merge return edges of e_i into P.right
        cdef _ConflictPair Q
        while True:
            Q = self.S.pop()
            if not Q.left.empty():
                Q.swap()
            if not Q.left.empty():  # not planar
                return False
            if self.lowpt[Q.right.low] > self.lowpt[e]:
                # merge intervals
                if P.right.empty():  # topmost interval
                    P.right = Q.right.copy()
                else:
                    self.ref[P.right.low] = Q.right.high
                P.right.low = Q.right.low
            else:  # align
                self.ref[Q.right.low] = self.lowpt_edge[e]
            if _top_of_stack(self.S) == self.stack_bottom[ei]:
                break

        # merge conflicting return edges of e_1,...,e_i-1 into P.L
        while (
                _top_of_stack(self.S).left.conflicting(ei, self) or
                _top_of_stack(self.S).right.conflicting(ei, self)
        ):
            Q = self.S.pop()
            if Q.right.conflicting(ei, self):
                Q.swap()
            if Q.right.conflicting(ei, self):  # not planar
                return False
            # merge interval below lowpt(e_i) into P.R
            self.ref[P.right.low] = Q.right.high
            if Q.right.low is not None:
                P.right.low = Q.right.low

            if P.left.empty():  # topmost interval
                P.left = Q.left.copy()
            else:
                self.ref[P.left.low] = Q.left.high
            P.left.low = Q.left.low

        if not (P.left.empty() and P.right.empty()):
            self.S.append(P)

        return True

    cdef void remove_back_edges(self, tuple e):
        cdef int u = e[0]

        # trim back edges ending at parent u
        # drop entire conflict pairs
        cdef _ConflictPair p
        while self.S and _top_of_stack(self.S).lowest(self) == self.height[u]:
            p = self.S.pop()
            if p.left.low is not None:
                self.side[p.left.low] = -1

        if self.S:  # one more conflict pair to consider
            p = self.S.pop()
            # trim left interval
            while p.left.high is not None and p.left.high[1] == u:
                p.left.high = self.ref[p.left.high]
            if p.left.high is None and p.left.low is not None:
                # just emptied
                self.ref[p.left.low] = p.right.low
                self.side[p.left.low] = -1
                p.left.low = None
            # trim right interval
            while p.right.high is not None and p.right.high[1] == u:
                p.right.high = self.ref[p.right.high]
            if p.right.high is None and p.right.low is not None:
                # just emptied
                self.ref[p.right.low] = p.left.low
                self.side[p.right.low] = -1
                p.right.low = None
            self.S.append(p)

        # side of e is side of a highest return edge
        if self.lowpt[e] < self.height[u]:  # e has return edge
            p = _top_of_stack(self.S)
            hl = p.left.high
            hr = p.right.high

            if (hl is not None) and (hr is None or self.lowpt[hl] > self.lowpt[hr]):
                self.ref[e] = hl
            else:
                self.ref[e] = hr

    cdef int sign(self, tuple e):
        """Resolve the relative side of an edge to the absolute side."""
        # the recursion stack
        dfs_stack = [e]
        # dict to remember reference edges
        old_ref = defaultdict(lambda: None)
        while dfs_stack:
            e = dfs_stack.pop()
            if self.ref[e] is not None:
                dfs_stack.append(e)  # revisit e after finishing self.ref[e]
                dfs_stack.append(self.ref[e])  # visit self.ref[e] next
                old_ref[e] = self.ref[e]  # remember value of self.ref[e]
                self.ref[e] = None
            else:
                self.side[e] *= self.side[old_ref[e]]
        return self.side[e]


cdef class _ConflictPair:

    """Represents a different constraint between two intervals.

    The edges in the left interval must have a different orientation than
    the one in the right interval.
    """

    cdef _Interval left, right

    def __cinit__(
        self,
        _Interval left=None,
        _Interval right=None
    ):
        if left is None:
            self.left = _Interval.__new__(_Interval)
        else:
            self.left = left
        if right is None:
            self.right = _Interval.__new__(_Interval)
        else:
            self.right = right

    cdef void swap(self):
        """Swap left and right intervals"""
        self.left, self.right = self.right, self.left

    cdef int lowest(self, _LRPlanarity planarity_state):
        """Return the lowest low point of a conflict pair"""
        if self.left.empty():
            return planarity_state.lowpt[self.right.low]
        if self.right.empty():
            return planarity_state.lowpt[self.left.low]
        return min(
            planarity_state.lowpt[self.left.low],
            planarity_state.lowpt[self.right.low]
        )


cdef class _Interval:

    """Represents a set of return edges.

    All return edges in an interval induce a same constraint on the contained
    edges, which means that all edges must either have a left orientation or
    all edges must have a right orientation.
    """

    cdef tuple low, high

    def __cinit__(self, tuple low=None, tuple high=None):
        self.low = low
        self.high = high

    cdef bint empty(self):
        """Check if the interval is empty"""
        return self.low is self.high is None

    cdef _Interval copy(self):
        """Return a copy of this interval"""
        return _Interval.__new__(_Interval, self.low, self.high)

    cdef bint conflicting(self, tuple b, _LRPlanarity state):
        """Return True if interval I conflicts with edge b"""
        return not self.empty() and state.lowpt[self.high] > state.lowpt[b]


cdef class _PlanarEmbedding(Graph):

    """Represents a planar graph with its planar embedding."""

    cdef cmap[int, int] node_label
    cdef cmap[ipair, ipair] edge_label

    cdef void add_half_edge_cw(self, int start_node, int end_node, int reference_neighbor):
        """Adds a half-edge from start_node to end_node.

        edge_label[:, :, :] 0: cw
        edge_label[:, :, :] 1: ccw
        node_label[:]: first_nbr
        """
        self.add_edge(start_node, end_node)  # Add edge to graph

        if reference_neighbor == -1:
            # The start node has no neighbors
            self.edge_label[ipair(start_node, end_node)].first = end_node
            self.edge_label[ipair(start_node, end_node)].second = end_node
            self.node_label[start_node] = end_node
            return

        # Get half-edge at the other side
        cdef int cw_reference = self.edge_label[ipair(start_node, reference_neighbor)].first
        # Alter half-edge data structures
        self.edge_label[ipair(start_node, reference_neighbor)].first = end_node
        self.edge_label[ipair(start_node, end_node)].first = cw_reference
        self.edge_label[ipair(start_node, cw_reference)].second = end_node
        self.edge_label[ipair(start_node, end_node)].second = reference_neighbor

    cdef void add_half_edge_ccw(self, int start_node, int end_node, int reference_neighbor):
        """Adds a half-edge from start_node to end_node."""
        self.add_half_edge_cw(
            start_node,
            end_node,
            self.edge_label[ipair(start_node, reference_neighbor)].second
        )
        cdef int s_f_label
        if self.node_label.find(start_node) != self.node_label.end():
            s_f_label = self.node_label[start_node]
        else:
            s_f_label = -1
        if reference_neighbor == s_f_label:
            # Update first neighbor
            self.node_label[start_node] = end_node

    cdef void add_half_edge_first(self, int start_node, int end_node):
        """The added half-edge is inserted at the first position in the order."""
        self.add_half_edge_ccw(start_node, end_node, self.node_label[start_node])



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Firefly Algorithm

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from libc.math cimport exp, log10, sqrt, HUGE_VAL
from .utility cimport (
    rand_v,
    Chromosome,
    Objective,
    AlgorithmBase,
)

ctypedef unsigned int uint


cdef double _distance(Chromosome me, Chromosome she, uint dim):
    """Distance of two fireflies."""
    cdef double dist = 0
    cdef uint i
    cdef double diff
    for i in range(dim):
        diff = me.v[i] - she.v[i]
        dist += diff * diff
    return sqrt(dist)


@cython.final
cdef class Firefly(AlgorithmBase):

    """Algorithm class."""

    cdef uint n
    cdef double alpha, alpha0, beta_min, beta0, gamma
    cdef Chromosome[:] fireflies

    def __cinit__(
        self,
        Objective func,
        dict settings,
        object progress_fun=None,
        object interrupt_fun=None
    ):
        """
        settings = {
            'n': int,
            'alpha': float,
            'beta_min': float,
            'beta0': float,
            'gamma': float,
            'max_gen': int or 'min_fit': float or 'max_time': float,
            'report': int,
        }
        """
        # n, the population size of fireflies
        self.n = settings.get('n', 80)
        # alpha, the step size
        self.alpha = settings.get('alpha', 0.01)
        # alpha0, use to calculate_new_alpha
        self.alpha0 = self.alpha
        # beta_min, the minimal attraction, must not less than this
        self.beta_min = settings.get('beta_min', 0.2)
        # beta0, the attraction of two firefly in 0 distance.
        self.beta0 = settings.get('beta0', 1.)
        # gamma
        self.gamma = settings.get('gamma', 1.)
        # all fireflies, depended on population n
        self.fireflies = Chromosome.new_pop(self.dim, self.n)

    cdef inline void initialize(self):
        cdef uint i, j
        cdef Chromosome tmp
        for i in range(self.n):
            # initialize the Chromosome
            tmp = self.fireflies[i]
            for j in range(self.dim):
                tmp.v[j] = rand_v(self.lb[j], self.ub[j])
        self.evaluate()
        self.last_best.assign(self.fireflies[0])

    cdef inline void move_fireflies(self):
        cdef uint i
        cdef bint is_move
        cdef double scale, tmp_v
        cdef Chromosome tmp, other
        for tmp in self.fireflies:
            is_move = False
            for other in self.fireflies:
                if tmp is other:
                    continue
                is_move |= self.move_firefly(tmp, other)
            if is_move:
                continue
            for i in range(self.dim):
                scale = self.ub[i] - self.lb[i]
                tmp_v = tmp.v[i] + self.alpha * scale * rand_v(-0.5, 0.5)
                tmp.v[i] = self.check(i, tmp_v)

    cdef inline void evaluate(self):
        cdef Chromosome firefly
        for firefly in self.fireflies:
            firefly.f = self.func.fitness(firefly.v)

    cdef inline bint move_firefly(self, Chromosome me, Chromosome she):
        if me.f <= she.f:
            return False
        cdef double r = _distance(me, she, self.dim)
        cdef double beta = (self.beta0 - self.beta_min) * exp(-self.gamma * r * r) + self.beta_min
        cdef uint i
        cdef double scale, me_v
        for i in range(self.dim):
            scale = self.ub[i] - self.lb[i]
            me_v = me.v[i] + beta * (she.v[i] - me.v[i]) + self.alpha * scale * rand_v(-0.5, 0.5)
            me.v[i] = self.check(i, me_v)
        return True

    cdef inline double check(self, int i, double v):
        if v > self.ub[i]:
            return self.ub[i]
        elif v < self.lb[i]:
            return self.lb[i]
        else:
            return v

    cdef inline Chromosome find_firefly(self):
        cdef int index = 0
        cdef double f = HUGE_VAL

        cdef int i
        cdef Chromosome tmp
        for i, tmp in enumerate(self.fireflies):
            tmp = self.fireflies[i]
            if tmp.f < f:
                index = i
                f = tmp.f
        return self.fireflies[index]

    cdef inline void generation_process(self):
        self.move_fireflies()
        self.evaluate()
        # adjust alpha, depended on fitness value
        # if fitness value is larger, then alpha should larger
        # if fitness value is small, then alpha should smaller
        cdef Chromosome current_best = self.find_firefly()
        if self.last_best.f > current_best.f:
            self.last_best.assign(current_best)
        self.alpha = self.alpha0 * log10(current_best.f + 1)



# -*- coding: utf-8 -*-

"""Kernel of Pyslvs.

This kernel can work without GUI.

Modules:
+ Solver:
    + parser
    + tinycadlib
        + Sketch Solve solver
    + triangulation
+ Number synthesis:
    + number
+ Structure Synthesis:
    + atlas
+ Dimensional synthesis:
    + planarlinkage
    + rga
    + firefly
    + de

Dependents:
+ lark-parser
"""

__author__ = "Yuan Chang"
__copyright__ = "Copyright (C) 2016-2019"
__license__ = "AGPL"
__email__ = "pyslvs@gmail.com"
__version__ = "19.12.0"

from .expression import get_vlinks, VJoint, VPoint, VLink, Coordinate
from .bfgs import SolverSystem
from .triangulation import vpoints_configure, ExpressionStack
from .tinycadlib import (
    plap,
    pllp,
    plpp,
    pxy,
    vpoint_dof,
    expr_parser,
    expr_solving,
    data_collecting,
)
from . import metaheuristics as _  # Preload module
from .planar_linkage import Planar
from .number import link_synthesis, contracted_link_synthesis
from .graph import (
    Graph,
    link_assortment,
    contracted_link_assortment,
    labeled_enumerate,
)
from .planar_check import is_planar
from .graph_layout import external_loop_layout
from .atlas import conventional_graph, contracted_graph
from .expression_parser import (
    color_names,
    color_rgb,
    parse_params,
    parse_pos,
    parse_vpoints,
    parse_vlinks,
    edges_view,
    graph2vpoints,
    PointArgs,
    LinkArgs,
)
from .example import example_list
from .collection import collection_list
from .efd import efd_fitting

__all__ = [
    '__version__',
    'Coordinate',
    'plap',
    'pllp',
    'plpp',
    'pxy',
    'expr_parser',
    'expr_solving',
    'data_collecting',
    'get_vlinks',
    'VJoint',
    'VPoint',
    'VLink',
    'SolverSystem',
    'Planar',
    'link_synthesis',
    'contracted_link_synthesis',
    'is_planar',
    'external_loop_layout',
    'conventional_graph',
    'contracted_graph',
    'Graph',
    'link_assortment',
    'contracted_link_assortment',
    'labeled_enumerate',
    'vpoints_configure',
    'ExpressionStack',
    'vpoint_dof',
    'color_names',
    'color_rgb',
    'parse_params',
    'parse_pos',
    'parse_vpoints',
    'parse_vlinks',
    'edges_view',
    'graph2vpoints',
    'PointArgs',
    'LinkArgs',
    'example_list',
    'collection_list',
    'efd_fitting',
]



# -*- coding: utf-8 -*-

"""The source code refer from "spatial_efd" module."""

__author__ = "Yuan Chang"
__copyright__ = "Copyright (C) 2016-2019"
__license__ = "AGPL"
__email__ = "pyslvs@gmail.com"

from typing import Tuple, Sized, Sequence
from math import pi, sin, cos, atan2, degrees, radians
from numpy import (
    sqrt,
    abs,
    cos as np_cos,
    sin as np_sin,
    dot,
    sum as np_sum,
    array,
    ndarray,
    linspace,
    zeros,
    ones,
    diff,
    concatenate,
    cumsum,
)


def efd_fitting(path: Sequence[Tuple[float, float]], n: int) -> ndarray:
    """Curve fitting using Elliptical Fourier Descriptor."""
    contour = array(path, dtype=float)
    harmonic = _fourier_power(
        _calculate_efd(contour, _nyquist(contour)),
        _nyquist(contour)
    )
    coeffs = _calculate_efd(contour, harmonic)
    coeffs, rotation = _normalize_efd(coeffs, size_invariant=False)
    locus = _calculate_dc_coefficients(contour)
    # New path
    contour = _inverse_transform(coeffs, locus, n, harmonic)
    contour = _rotate_contour(contour, -rotation, locus)
    return contour


def _normalize_efd(
    coeffs: ndarray,
    size_invariant: bool = True
) -> Tuple[ndarray, float]:
    """
    Normalize the Elliptical Fourier Descriptor coefficients for a polygon.

    Implements Kuhl and Giardina method of normalizing the coefficients
    An, Bn, Cn, Dn. Performs 3 separate normalizations. First, it makes the
    data location invariant by re-scaling the data to a common origin.
    Secondly, the data is rotated with respect to the major axis. Thirdly,
    the coefficients are normalized with regard to the absolute value of A_1.
    This code is adapted from the pyefd module. See the original paper for
    more detail:

    Kuhl, FP and Giardina, CR (1982). Elliptic Fourier features of a closed
    contour. Computer graphics and image procesnp_sing, 18(3), 236-258.

    Args:
        coeffs: A numpy array of shape (n, 4) representing the
            four coefficients for each harmonic computed.
        size_invariant: Set to True (the default) to perform the third
            normalization and false to return the data withot this procesnp_sing
            step. Set this to False when plotting a comparison between the
            input data and the Fourier ellipse.
    Returns:
        A tuple consisting of a numpy.ndarray of shape (harmonics, 4)
        representing the four coefficients for each harmonic computed and
        the rotation in degrees applied to the normalized contour.
    """
    # Make the coefficients have a zero phase shift from
    # the first major axis. Theta_1 is that shift angle.
    theta_1 = atan2(
        2 * (coeffs[0, 0] * coeffs[0, 1] + coeffs[0, 2] * coeffs[0, 3]),
        coeffs[0, 0] ** 2 - coeffs[0, 1] ** 2 + coeffs[0, 2] ** 2 - coeffs[0, 3] ** 2
    ) * 0.5
    # Rotate all coefficients by theta_1.
    for n in range(coeffs.shape[0]):
        angle = (n + 1) * theta_1
        coeffs[n, :] = dot(
            array([
                [coeffs[n, 0], coeffs[n, 1]],
                [coeffs[n, 2], coeffs[n, 3]],
            ]),
            array([
                [np_cos(angle), -np_sin(angle)],
                [np_sin(angle), np_cos(angle)],
            ])
        ).flatten()
    # Make the coefficients rotation invariant by rotating so that
    # the semi-major axis is parallel to the x-axis.
    psi_1 = atan2(coeffs[0, 2], coeffs[0, 0])
    psi_r = array([
        [np_cos(psi_1), np_sin(psi_1)],
        [-np_sin(psi_1), np_cos(psi_1)],
    ])
    # Rotate all coefficients by -psi_1.
    for n in range(coeffs.shape[0]):
        rot = array([
            [coeffs[n, 0], coeffs[n, 1]],
            [coeffs[n, 2], coeffs[n, 3]],
        ])
        coeffs[n, :] = psi_r.dot(rot).flatten()
    if size_invariant:
        # Obtain size-invariance by normalizing.
        coeffs /= abs(coeffs[0, 0])
    return coeffs, degrees(psi_1)


def _calculate_dc_coefficients(contour: ndarray) -> Tuple[float, float]:
    """
    Compute the dc coefficients, used as the locus when calling
    inverse_transform().

    This code is adapted from the pyefd module. See the original paper for
    more detail:

    Kuhl, FP and Giardina, CR (1982). Elliptic Fourier features of a closed
    contour. Computer graphics and image procesnp_sing, 18(3), 236-258.

    Args:
        contour: A n x 2 numpy array represents a path.
    Returns:
        A tuple containing the c and d coefficients.
    """
    dxy = diff(contour, axis=0)
    dt = sqrt((dxy ** 2).sum(axis=1))
    t = concatenate(([0], cumsum(dt)))
    zt = t[-1]
    diffs = diff(t ** 2)
    xi = cumsum(dxy[:, 0]) - dxy[:, 0] / dt * t[1:]
    a0 = 1 / zt * np_sum(dxy[:, 0] / (2 * dt) * diffs + xi * dt)
    delta = cumsum(dxy[:, 1]) - dxy[:, 1] / dt * t[1:]
    c0 = 1 / zt * np_sum(dxy[:, 1] / (2 * dt) * diffs + delta * dt)
    # A0 and CO relate to the first point of the contour array as origin.
    # Adding those values to the coefficients to make them relate to true origin
    return contour[0, 0] + a0, contour[0, 1] + c0


def _calculate_efd(contour: ndarray, harmonic: int = 10) -> ndarray:
    """
    Compute the Elliptical Fourier Descriptors for a polygon.

    Implements Kuhl and Giardina method of computing the coefficients
    An, Bn, Cn, Dn for a specified number of harmonics. This code is adapted
    from the pyefd module. See the original paper for more detail:

    Kuhl, FP and Giardina, CR (1982). Elliptic Fourier features of a closed
    contour. Computer graphics and image procesnp_sing, 18(3), 236-258.

    Args:
        contour: A n x 2 numpy array represents a path.
        harmonic: The number of harmonics to compute for the given
            shape, defaults to 10.
    Returns:
        A numpy array of shape (harmonics, 4) representing the
        four coefficients for each harmonic computed.
    """
    dxy = diff(contour, axis=0)
    dt = sqrt((dxy ** 2).sum(axis=1))
    t = concatenate(([0], cumsum(dt)))
    zt = t[-1]
    phi = (2. * pi * t) / zt
    coeffs = zeros((harmonic, 4))
    for n in range(1, harmonic + 1):
        const = zt / (2 * n * n * pi * pi)
        phi_n = phi * n
        d_np_cos_phi_n = np_cos(phi_n[1:]) - np_cos(phi_n[:-1])
        d_np_sin_phi_n = np_sin(phi_n[1:]) - np_sin(phi_n[:-1])
        coeffs[n - 1, :] = (
            const * np_sum(dxy[:, 1] / dt * d_np_cos_phi_n),
            const * np_sum(dxy[:, 1] / dt * d_np_sin_phi_n),
            const * np_sum(dxy[:, 0] / dt * d_np_cos_phi_n),
            const * np_sum(dxy[:, 0] / dt * d_np_sin_phi_n),
        )
    return coeffs


def _inverse_transform(
    coeffs: ndarray,
    locus: Tuple[float, float] = (0., 0.),
    n: int = 300,
    harmonic: int = 10
) -> ndarray:
    """
    Perform an inverse fourier transform to convert the coefficients back into
    spatial coordinates.

    Implements Kuhl and Giardina method of computing the performing the
    transform for a specified number of harmonics. This code is adapted
    from the pyefd module. See the original paper for more detail:

    Kuhl, FP and Giardina, CR (1982). Elliptic Fourier features of a closed
    contour. Computer graphics and image procesnp_sing, 18(3), 236-258.

    Args:
        coeffs: A numpy array of shape (n, 4) representing the
            four coefficients for each harmonic computed.
        locus: The x,y coordinates of the centroid of the contour being
            generated. Use calculate_dc_coefficients() to generate the correct
            locus for a shape.
        n: The number of coordinate pairs to compute. A larger value will
            result in a more complex shape at the expense of increased
            computational time. Defaults to 300.
        harmonic: The number of harmonics to be used to generate
            coordinates, defaults to 10. Must be <= coeffs.shape[0]. Supply a
            smaller value to produce coordinates for a more generalized shape.
    Returns:
        A n x 2 numpy array represents a contour.
    """
    t = linspace(0, 1, n)
    contour = ones((n, 2), dtype=float)
    contour[:, 0] *= locus[0]
    contour[:, 1] *= locus[1]
    for n in range(harmonic):
        angle = 2 * (n + 1) * pi * t
        cosine = np_cos(angle)
        sine = np_sin(angle)
        contour[:, 0] += coeffs[n, 2] * cosine + coeffs[n, 3] * sine
        contour[:, 1] += coeffs[n, 0] * cosine + coeffs[n, 1] * sine
    return contour


def _nyquist(zx: Sized) -> int:
    """
    Returns the maximum number of harmonics that can be computed for a given
    contour, the Nyquist Frequency.

    See this paper for details:
    C. np_costa et al. / Postharvest Biology and Technology 54 (2009) 38-47

    Args:
        zx (list): A list (or numpy array) of x coordinate values.
    Returns:
        int: The nyquist frequency, expressed as a number of harmonics.
    """
    return len(zx) // 2


def _fourier_power(
    coeffs: ndarray,
    nyq: int,
    threshold: float = 0.9999
) -> int:
    """
    Compute the total Fourier power and find the minium number of harmonics
    required to exceed the threshold fraction of the total power.

    This is a good method for identifying the number of harmonics to use to
    describe a polygon. For more details see:

    C. np_costa et al. / Postharvest Biology and Technology 54 (2009) 38-47

    Warning:
        The number of coefficients must be >= the Nyquist Frequency.
    Args:
        coeffs: A numpy array of shape (n, 4) representing the
            four coefficients for each harmonic computed.
        nyq: The Nyquist Frequency.
        threshold: The threshold fraction of the total Fourier power,
            the default is 0.9999.
    Returns:
        The number of harmonics required to represent the contour above
        the threshold Fourier power.
    """
    total_power = 0
    current_power = 0
    for i in range(nyq):
        total_power += 0.5 * np_sum(coeffs[i, :] ** 2)
    for i in range(nyq):
        current_power += 0.5 * np_sum(coeffs[i, :] ** 2)
        if current_power / total_power > threshold:
            return i + 1
    return nyq


def _rotate_contour(
    contour: ndarray,
    rotation: float,
    centroid: Tuple[float, float]
) -> ndarray:
    """
    Rotates a contour about a point by a given amount expressed in degrees.

    Operates by calling rotatePoint() on each x,y pair in turn. X and Y must
    have the same dimensions.

    Args:
        contour: A n x 2 numpy array represents a path.
        rotation: The angle in degrees for the contour to be rotated by.
        centroid: A tuple containing the x,y coordinates of the centroid to
            rotate the contour about.
    Returns:
        A n x 2 numpy array represents a contour.
    """
    angle = radians(rotation)
    cpx, cpy = centroid
    new_contour = zeros(contour.shape, dtype=contour.dtype)
    for i in range(len(contour)):
        x, y = contour[i]
        dx = x - cpx
        dy = y - cpy
        new_contour[i] = (
            cpx + dx * cos(angle) - dy * sin(angle),
            cpy + dx * sin(angle) + dy * cos(angle)
        )
    return new_contour



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Tiny CAD library of PMKS symbolic and position analysis.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from libc.math cimport M_PI, sqrt, sin, cos, atan2, NAN
from .expression cimport VJoint, VPoint, VLink
from .triangulation cimport (
    symbol,
    symbol_str,
    Expression,
    PLA,
    PLAP,
    PLLP,
    PLPP,
    PXY,
)
from .bfgs cimport SolverSystem

cdef Coordinate _NAN_COORD = Coordinate.__new__(Coordinate, NAN, NAN)


cdef inline double radians(double degree) nogil:
    """Deg to rad."""
    return degree / 180 * M_PI


cpdef Coordinate plap(
    Coordinate c1,
    double d0,
    double a0,
    Coordinate c2 = None,
    bint inverse = False
):
    """Point on circle by angle."""
    cdef double a1 = atan2(c2.y - c1.y, c2.x - c1.x) if c2 is not None else 0
    if inverse:
        a1 -= a0
    else:
        a1 += a0
    return Coordinate.__new__(Coordinate, c1.x + d0 * cos(a1), c1.y + d0 * sin(a1))


cpdef Coordinate pllp(
    Coordinate c1,
    double d0,
    double d1,
    Coordinate c2,
    bint inverse = False
):
    """Two intersection points of two circles."""
    cdef double dx = c2.x - c1.x
    cdef double dy = c2.y - c1.y
    cdef double d = c1.distance(c2)

    # No solutions, the circles are separate
    if d > d0 + d1:
        return _NAN_COORD

    # No solutions because one circle is contained within the other
    if d < abs(d0 - d1):
        return _NAN_COORD

    # Circles are coincident and there are an infinite number of solutions
    if d == 0 and d0 == d1:
        return _NAN_COORD
    cdef double a = (d0 * d0 - d1 * d1 + d * d) / (2 * d)
    cdef double h = sqrt(d0 * d0 - a * a)
    cdef double xm = c1.x + a * dx / d
    cdef double ym = c1.y + a * dy / d

    if inverse:
        return Coordinate.__new__(Coordinate, xm + h * dy / d, ym - h * dx / d)
    else:
        return Coordinate.__new__(Coordinate, xm - h * dy / d, ym + h * dx / d)


cpdef Coordinate plpp(
    Coordinate c1,
    double d0,
    Coordinate c2,
    Coordinate c3,
    bint inverse = False
):
    """Two intersection points of a line and a circle."""
    cdef double line_mag = c2.distance(c3)
    cdef double dx = c3.x - c2.x
    cdef double dy = c3.y - c2.y
    cdef double u = ((c1.x - c2.x) * dx + (c1.y - c2.y) * dy) / (line_mag * line_mag)
    cdef Coordinate inter = Coordinate.__new__(Coordinate, c2.x + u * dx, c2.y + u * dy)

    # Test distance between point A and intersection
    cdef double d = c1.distance(inter)
    if d > d0:
        # No intersection
        return _NAN_COORD
    elif d == d0:
        # One intersection point
        return inter.x, inter.y

    # Two intersection points
    d = sqrt(d0 * d0 - d * d) / line_mag
    dx *= d
    dy *= d
    if inverse:
        return Coordinate.__new__(Coordinate, inter.x - dx, inter.y - dy)
    else:
        return Coordinate.__new__(Coordinate, inter.x + dx, inter.y + dy)


cpdef Coordinate pxy(Coordinate c1, double x, double y):
    """Using relative cartesian coordinate to get solution."""
    return Coordinate.__new__(Coordinate, c1.x + x, c1.y + y)


cdef inline str str_between(str s, str front, str back):
    """Get the string that is inside of parenthesis."""
    return s[s.find(front) + 1:s.find(back)]


cdef inline str str_before(str s, str front):
    """Get the string that is front of parenthesis."""
    return s[:s.find(front)]


cpdef void expr_parser(ExpressionStack exprs, dict data_dict):
    """Update data.

    + exprs: [("PLAP", "P0", "L0", "a0", "P1", "P2"), ..."]
    + data_dict: {'a0':0., 'L1':10., 'A':(30., 40.), ...}
    """
    cdef symbol target
    cdef Coordinate coord, coord1, coord2, coord3
    cdef Expression expr
    for expr in exprs.stack:
        coord = _NAN_COORD
        if expr.func in {PLA, PLAP}:
            coord1 = data_dict[symbol_str(expr.c1)]
            if expr.func == PLA:
                target = expr.c2
                coord = plap(
                    coord1,
                    data_dict[symbol_str(expr.v1)],
                    data_dict[symbol_str(expr.v2)]
                )
            else:
                target = expr.c3
                coord2 = data_dict[symbol_str(expr.c2)]
                coord = plap(
                    coord1,
                    data_dict[symbol_str(expr.v1)],
                    data_dict[symbol_str(expr.v2)],
                    coord2,
                    expr.op
                )
        elif expr.func == PLLP:
            target = expr.c3
            coord1 = data_dict[symbol_str(expr.c1)]
            coord2 = data_dict[symbol_str(expr.c2)]
            coord = pllp(
                coord1,
                data_dict[symbol_str(expr.v1)],
                data_dict[symbol_str(expr.v2)],
                coord2,
                expr.op
            )
        elif expr.func == PLPP:
            target = expr.c4
            coord1 = data_dict[symbol_str(expr.c1)]
            coord2 = data_dict[symbol_str(expr.c2)]
            coord3 = data_dict[symbol_str(expr.c3)]
            coord = plpp(
                coord1,
                data_dict[symbol_str(expr.v1)],
                coord2,
                coord3,
                expr.op
            )
        elif expr.func == PXY:
            target = expr.c2
            coord1 = data_dict[symbol_str(expr.c1)]
            coord = pxy(
                coord1,
                data_dict[symbol_str(expr.v1)],
                data_dict[symbol_str(expr.v2)]
            )
        else:
            raise ValueError("unsupported function")
        data_dict[symbol_str(target)] = coord


cpdef int vpoint_dof(object vpoints):
    """Degree of freedoms calculate from PMKS expressions."""
    # Joint with DOF 1
    cdef int j1 = 0
    # Joint with DOF 2
    cdef int j2 = 0
    # First link is frame
    vlinks = {VLink.FRAME}
    cdef int link_count
    cdef VPoint vpoint
    for vpoint in vpoints:
        link_count = len(vpoint.links)
        if not link_count > 1:
            # If a point doesn't have two more links, it can not be call a 'joint'.
            continue
        vlinks.update(vpoint.links)
        if vpoint.type == VJoint.R:
            j1 += link_count - 1
        elif vpoint.type == VJoint.P:
            if link_count > 2:
                j1 += link_count - 2
            j1 += 1
        elif vpoint.type == VJoint.RP:
            if link_count > 2:
                j1 += link_count - 2
            j2 += 1
    return 3 * (len(vlinks) - 1) - (2 * j1) - j2


cdef inline int base_friend(int node, object vpoints):
    cdef int i
    cdef VPoint vpoint
    for i, vpoint in enumerate(vpoints):
        if not vpoints[node].links:
            continue
        if vpoints[node].links[0] in vpoint.links:
            return i


cpdef tuple data_collecting(ExpressionStack exprs, dict mapping, object vpoints_):
    """Data collecting process.

    Input data:
    + exprs: [('PLAP', 'P0', 'L0', 'a0', 'P1', 'P2'), ...]
    + mapping: {0: 'P0', 1: 'P2', 2: 'P3', 3: 'P4', ...}
        + Specify position: {'P0': (10., 20.)}
        + Specify link length: {(a, b): 20.}
    + vpoints_: [VPoint0, VPoint1, VPoint2, ...]
    + pos: [(x0, y0), (x1, y1), (x2, y2), ...]

    vpoints will make a copy that we don't want to modified origin data.
    """
    vpoints = list(vpoints_)
    # First, we create a "VLinks" that can help us to
    # find a relationship just like adjacency matrix.
    cdef int node
    cdef VPoint vpoint
    vlinks = {}
    for node, vpoint in enumerate(vpoints):
        for link in vpoint.links:
            # Add as vlink.
            if link not in vlinks:
                vlinks[link] = [node]
            else:
                vlinks[link].append(node)

    # Replace the P joints and their friends with RP joint.
    # DOF must be same after properties changed.
    cdef int base
    cdef double x, y
    cdef VPoint vpoint_
    links = set()
    for base in range(len(vpoints)):
        vpoint = vpoints[base]
        if vpoint.type != VJoint.P:
            continue
        for link in vpoint.links[1:]:
            links.clear()
            for node in vlinks[link]:
                vpoint_ = vpoints[node]
                if node == base or vpoint_.type in {VJoint.P, VJoint.RP}:
                    continue
                links.update(vpoint_.links)
                x, y = vpoint_.c[0]
                vpoints[node] = VPoint.c_slider_joint(
                    [vpoint.links[0]] + [
                        link_ for link_ in vpoint_.links
                        if (link_ not in vpoint.links)
                    ],
                    VJoint.RP,
                    vpoint.angle,
                    x,
                    y
                )

    # Reverse mapping, exclude specified link length.
    mapping_r = {}
    length = {}
    data_dict = {}
    for k, v in mapping.items():
        if type(k) is int:
            mapping_r[v] = k
            if v in mapping:
                x, y = mapping[v]
                data_dict[v] = Coordinate.__new__(Coordinate, x, y)
        elif type(k) is tuple:
            length[frozenset(k)] = v

    pos = []
    for vpoint in vpoints:
        if vpoint.type == VJoint.R:
            pos.append(Coordinate.__new__(Coordinate, vpoint.c[0][0], vpoint.c[0][1]))
        else:
            pos.append(Coordinate.__new__(Coordinate, vpoint.c[1][0], vpoint.c[1][1]))

    cdef int i, bf
    cdef double angle
    # Add slider slot virtual coordinates.
    for i, vpoint in enumerate(vpoints):
        # PLPP dependents.
        if vpoint.type != VJoint.RP:
            continue
        bf = base_friend(i, vpoints)
        angle = radians(
            vpoint.angle -
            vpoint.slope_angle(vpoints[bf], 1, 0) +
            vpoint.slope_angle(vpoints[bf], 0, 0)
        )
        pos.append(Coordinate.__new__(Coordinate, vpoint.c[1][0] + cos(angle), vpoint.c[1][1] + sin(angle)))
        mapping_r[f'S{i}'] = len(pos) - 1

    # Add data to 'data_dict' and counting DOF.
    cdef int dof = 0
    cdef int target
    cdef Expression expr
    cdef Coordinate coord1, coord2
    for expr in exprs.stack:
        node = mapping_r[symbol_str(expr.c1)]

        # Point 1
        if symbol_str(expr.c1) not in data_dict:
            data_dict[symbol_str(expr.c1)] = pos[mapping_r[symbol_str(expr.c1)]]

        if expr.func in {PLA, PLAP}:
            if expr.func == PLA:
                target = mapping_r[symbol_str(expr.c2)]
            else:
                target = mapping_r[symbol_str(expr.c3)]
            # Link 1
            pair = frozenset({node, target})
            if pair in length:
                data_dict[symbol_str(expr.v1)] = length[pair]
            else:
                coord1 = pos[node]
                coord2 = pos[target]
                data_dict[symbol_str(expr.v1)] = coord1.distance(coord2)
            # Point 2
            if expr.func == PLAP and symbol_str(expr.c2) not in data_dict:
                data_dict[symbol_str(expr.c2)] = pos[mapping_r[symbol_str(expr.c2)]]
            # Inputs
            dof += 1
        elif expr.func == PLLP:
            target = mapping_r[symbol_str(expr.c3)]
            # Link 1
            pair = frozenset({node, target})
            if pair in length:
                data_dict[symbol_str(expr.v1)] = length[pair]
            else:
                coord1 = pos[node]
                coord2 = pos[target]
                data_dict[symbol_str(expr.v1)] = coord1.distance(coord2)
            # Link 2
            pair = frozenset({mapping_r[symbol_str(expr.c2)], target})
            if pair in length:
                data_dict[symbol_str(expr.v2)] = length[pair]
            else:
                coord1 = pos[mapping_r[symbol_str(expr.c2)]]
                coord2 = pos[target]
                data_dict[symbol_str(expr.v2)] = coord1.distance(coord2)
            # Point 2
            if symbol_str(expr.c2) not in data_dict:
                data_dict[symbol_str(expr.c2)] = pos[mapping_r[symbol_str(expr.c2)]]
        elif expr.func == PLPP:
            target = mapping_r[symbol_str(expr.c4)]
            # Link 1
            pair = frozenset({node, target})
            if pair in length:
                data_dict[symbol_str(expr.v1)] = length[pair]
            else:
                coord1 = pos[node]
                coord2 = pos[target]
                data_dict[symbol_str(expr.v1)] = coord1.distance(coord2)
            # Point 2
            if symbol_str(expr.c2) not in data_dict:
                data_dict[symbol_str(expr.c2)] = pos[mapping_r[symbol_str(expr.c2)]]
        elif expr.func == PXY:
            target = mapping_r[symbol_str(expr.c2)]
            coord1 = pos[node]
            coord2 = pos[target]
            # X
            if symbol_str(expr.v1) in mapping:
                x, y = mapping[symbol_str(expr.v1)]
                data_dict[symbol_str(expr.v1)] = Coordinate.__new__(Coordinate, x, y)
            else:
                data_dict[symbol_str(expr.v1)] = coord2.x - coord1.x
            # Y
            if symbol_str(expr.v2) in mapping:
                x, y = mapping[symbol_str(expr.v2)]
                data_dict[symbol_str(expr.v2)] = Coordinate.__new__(Coordinate, x, y)
            else:
                data_dict[symbol_str(expr.v2)] = coord2.y - coord1.y

    # Other grounded R joints.
    for i, vpoint in enumerate(vpoints):
        if vpoint.grounded() and vpoint.type == VJoint.R:
            x, y = vpoint.c[0]
            data_dict[mapping[i]] = Coordinate.__new__(Coordinate, x, y)

    return data_dict, dof


cpdef list expr_solving(
    ExpressionStack exprs,
    dict mapping,
    object vpoints,
    object angles = None
):
    """Solving function.
    
    + exprs: [('PLAP', 'P0', 'L0', 'a0', 'P1'), ...]
    + mapping: {0: 'P0', ..., (0, 1): 20.0, ...}
    + vpoints: [VPoint]
    + angles: [[a0]: a0, [a1]: a1, ...]
    """
    # Blank sequences
    if angles is None:
        angles = []

    data_dict, dof_input = data_collecting(exprs, mapping, vpoints)
    # Check input number
    cdef int dof = vpoint_dof(vpoints)
    if dof_input > dof:
        raise ValueError(
            f"wrong number of input parameters: {dof_input} / {dof}"
        )

    # Reverse mapping, exclude specified link length
    mapping_r = {v: k for k, v in mapping.items() if type(k) is int}

    # Check input pairs
    cdef int target
    cdef Expression expr
    for expr in exprs.stack:
        if expr.func in {PLA, PLAP}:
            if expr.func == PLA:
                target = mapping_r[symbol_str(expr.c2)]
            else:
                target = mapping_r[symbol_str(expr.c3)]
            if (
                vpoints[mapping_r[symbol_str(expr.c1)]].grounded()
                and vpoints[target].grounded()
            ):
                raise ValueError("wrong driver definition.")

    # Angles.
    cdef double a
    cdef int i
    for i, a in enumerate(angles):
        data_dict[f'a{i}'] = radians(a)

    # Solve
    if not exprs.stack.empty():
        expr_parser(exprs, data_dict)

    p_data_dict = {}
    cdef bint has_not_solved = False

    # Add coordinate of known points.
    for i in range(len(vpoints)):
        # {1: 'A'} vs {'A': (10., 20.)}
        if mapping[i] in data_dict:
            p_data_dict[i] = data_dict[mapping[i]]
        else:
            has_not_solved = True

    # Calling Sketch Solve kernel and try to get the result.
    if has_not_solved:
        # Add specified link lengths.
        for k, v in mapping.items():
            if type(k) is tuple:
                p_data_dict[k] = v
        # Solve
        try:
            solved_bfgs = SolverSystem(vpoints, {}, p_data_dict).solve()
        except ValueError:
            raise ValueError("result contains failure from sketch solve")
    # Format:
    # R joint: [[p0]: (p0_x, p0_y), [p1]: (p1_x, p1_y)]
    # P or RP joint: [[p2]: ((p2_x0, p2_y0), (p2_x1, p2_y1))]
    solved_points = []
    cdef VPoint vpoint
    cdef Coordinate coord
    for i in range(len(vpoints)):
        vpoint = vpoints[i]
        if mapping[i] in data_dict:
            # These points has been solved.
            coord = data_dict[mapping[i]]
            if coord.is_nan():
                raise ValueError(f"result contains failure: Point{i}")
            if vpoint.type == VJoint.R:
                solved_points.append((coord.x, coord.y))
            else:
                solved_points.append((vpoint.c[0], (coord.x, coord.y)))
        elif solved_bfgs is not None:
            # These points solved by Sketch Solve.
            if vpoint.type == VJoint.R:
                solved_points.append(solved_bfgs[i])
            else:
                solved_points.append((solved_bfgs[i][0], solved_bfgs[i][1]))
        else:
            # No answer.
            if vpoint.type == VJoint.R:
                solved_points.append(vpoint.c[0])
            else:
                solved_points.append(vpoint.c)
    return solved_points



# -*- coding: utf-8 -*-
# cython: language_level=3

"""Wrapper header of BFGS algorithm.
If you need to use container to carry floating point data,
std::list is recommended instead of std::vector.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cdef extern from "bfgs_solver/solve.h" nogil:

    int Rough
    int Fine
    int MaxIterations

    int Success
    int NoSolution

    struct Point:
        double *x
        double *y

    struct Line:
        Point *p1
        Point *p2

    struct Arc:
        double *startAngle
        double *endAngle
        double *rad
        Point *center

    struct Circle:
        Point *center
        double *rad

    struct Constraint:
        int type
        Point *point1
        Point *point2
        Line *line1
        Line *line2
        Line *SymLine
        Circle *circle1
        Circle *circle2
        Arc *arc1
        Arc *arc2
        double *parameter

    Constraint PointOnPointConstraint(Point *, Point *);
    Constraint P2PDistanceConstraint(Point *, Point *, double *)
    Constraint P2PDistanceVertConstraint(Point *, Point *, double *)
    Constraint P2PDistanceHorzConstraint(Point *, Point *, double *)
    Constraint PointOnLineConstraint(Point *, Line *)
    Constraint P2LDistanceConstraint(Point *, Line *, double *)
    Constraint P2LDistanceVertConstraint(Point *, Line *, double *)
    Constraint P2LDistanceHorzConstraint(Point *, Line *, double *)
    Constraint VerticalConstraint(Line *)
    Constraint HorizontalConstraint(Line *)
    Constraint TangentToCircleConstraint(Line *, Circle *)
    Constraint TangentToArcConstraint(Line *, Arc *)
    Constraint ArcRulesConstraint(Arc *)
    Constraint LineLengthConstraint(Line *)
    Constraint EqualLegnthConstraint(Line *, Line *)
    Constraint ArcRadiusConstraint(Arc *, double *)
    Constraint EqualRadiusArcsConstraint(Arc *, Arc *)
    Constraint EqualRadiusCirclesConstraint(Circle *, Circle *)
    Constraint EqualRadiusCircArcConstraint(Circle *, Arc *)
    Constraint ConcentricArcsConstraint(Arc *, Arc *)
    Constraint ConcentricCirclesConstraint(Circle *, Circle *)
    Constraint ConcentricCircArcConstraint(Circle *, Arc *)
    Constraint CircleRadiusConstraint(Circle *, double *)
    Constraint InternalAngleConstraint(Line *, Line *, double *)
    Constraint ExternalAngleConstraint(Line *, Line *, double *)
    Constraint PerpendicularConstraint(Line *, Line *)
    Constraint ParallelConstraint(Line *, Line *)
    Constraint ColinearConstraint(Line *, Line *)
    Constraint PointOnCircleConstraint(Point *, Circle *)
    Constraint PointOnArcConstraint(Point *, Arc *)
    Constraint PointOnLineMidpointConstraint(Point *, Line *)
    Constraint PointOnArcMidpointConstraint(Point *, Arc *)
    Constraint PointOnCircleQuadConstraint(Point *, Circle *, double *)
    Constraint SymmetricPointsConstraint(Point *, Point *, Line *)
    Constraint SymmetricLinesConstraint(Line *, Line *, Line *)
    Constraint SymmetricCirclesConstraint(Circle *, Circle *, Line *)
    Constraint SymmetricArcsConstraint(Arc *, Arc *, Line *)
    Constraint LineInternalAngleConstraint(Line *, double *)
    Constraint LineExternalAngleConstraint(Line *, double *)

    int solve(double **, const size_t, Constraint *, const size_t, int)
    void derivatives(double **, double *, int, Constraint *, int)



# -*- coding: utf-8 -*-

"""Kernel of Metaheuristic Algorithm."""

from typing import Dict, Union, Type
from enum import unique, Enum
from .utility import Objective, AlgorithmBase
from .rga import Genetic
from .firefly import Firefly
from .de import Differential
from .tlbo import TeachingLearning

__all__ = [
    'Objective',
    'AlgorithmBase',
    'AlgorithmType',
    'ALGORITHM',
    'PARAMS',
    'DEFAULT_PARAMS',
]


@unique
class AlgorithmType(str, Enum):

    """Enum type of algorithms."""

    RGA = "Real-coded Genetic Algorithm"
    Firefly = "Firefly Algorithm"
    DE = "Differential Evolution"
    TLBO = "Teaching Learning Based Optimization"


ALGORITHM: Dict[AlgorithmType, Type[AlgorithmBase]] = {
    AlgorithmType.RGA: Genetic,
    AlgorithmType.Firefly: Firefly,
    AlgorithmType.DE: Differential,
    AlgorithmType.TLBO: TeachingLearning,
}
PARAMS: Dict[AlgorithmType, Dict[str, Union[int, float]]] = {
    AlgorithmType.RGA: {
        'nPop': 500,
        'pCross': 0.95,
        'pMute': 0.05,
        'pWin': 0.95,
        'bDelta': 5.,
    },
    AlgorithmType.Firefly: {
        'n': 80,
        'alpha': 0.01,
        'beta_min': 0.2,
        'gamma': 1.,
        'beta0': 1.,
    },
    AlgorithmType.DE: {
        'strategy': 1,
        'NP': 400,
        'F': 0.6,
        'CR': 0.9,
    },
    AlgorithmType.TLBO: {
        'class_size': 50,
    }
}
DEFAULT_PARAMS = {'max_gen': 1000, 'report': 50}



# -*- coding: utf-8 -*-

from typing import Dict, Any
from numpy import ndarray
from .metaheuristics.utility import Objective

class Planar(Objective[str]):

    """This class is used to verified kinematics of the linkage mechanism."""

    def __init__(self, mech: Dict[str, Any]) -> None:
        """mech = {
            'expression': List[VPoint],
            'input': OrderedDict([((b0, d0), [start, end]), ...]),
            'placement': {pt: (x, y, r)},
            'target': {pt: [(x0, y0), (x1, y1), ...]},
            'same': {pt: match_to_pt},
            # Bound has no position data.
            'upper': List[float],
            'lower': List[float],
        }
        """
        ...

    def get_upper(self) -> ndarray:
        ...

    def get_lower(self) -> ndarray:
        ...

    def is_two_kernel(self) -> bool:
        ...

    def result(self, v: ndarray) -> str:
        """Return the last answer."""
        ...



# -*- coding: utf-8 -*-
# cython: language_level=3

"""The callable class of the validation in algorithm.
The 'utility' module should be loaded when using sub-class.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

ctypedef unsigned int uint

cdef enum stop_option:
    MAX_GEN
    MIN_FIT
    MAX_TIME
    SLOW_DOWN

cdef double rand_v(double lower = *, double upper = *) nogil
cdef uint rand_i(int upper) nogil


cdef class Chromosome:
    cdef double f
    cdef double[:] v
    cdef void assign(self, Chromosome obj)
    @staticmethod
    cdef Chromosome[:] new_pop(uint d, uint n)


cdef class Objective:
    cpdef double[:] get_upper(self)
    cpdef double[:] get_lower(self)
    cdef double fitness(self, double[:] v)
    cpdef object result(self, double[:] v)


cdef class AlgorithmBase:

    cdef public Objective func
    cdef uint dim, stop_at_i, gen, rpt
    cdef double stop_at_f, time_start
    cdef stop_option stop_at
    cdef Chromosome last_best
    cdef list fitness_time
    cdef double[:] lb, ub
    cdef object progress_fun, interrupt_fun

    cdef void initialize(self)
    cdef void generation_process(self)
    cdef void report(self)
    cpdef list history(self)
    cpdef object run(self)



# -*- coding: utf-8 -*-

from typing import Tuple, List, Sequence, Dict, Optional
from .expression import VPoint

class ExpressionStack:

    """The stack of Python wrapper."""

    def as_list(self) -> List[Tuple[str, ...]]:
        """Turn into normal list."""
        ...

    def __repr__(self) -> str:
        ...

def vpoints_configure(
    vpoints_: Sequence[VPoint],
    inputs: Sequence[Tuple[int, int]],
    status: Optional[Dict[int, bool]] = None
) -> ExpressionStack:
    """Auto configuration algorithm."""
    ...



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""The callable class of the validation in algorithm.
The 'utility' module should be loaded when using sub-class of base classes.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from time import process_time
from numpy import zeros, array as np_array, float64 as np_float
cimport cython
from libc.stdlib cimport rand, srand, RAND_MAX
from libc.time cimport time


cdef inline double rand_v(double lower = 0., double upper = 1.) nogil:
    """Random real value between lower <= r <= upper."""
    return lower + <double>rand() / RAND_MAX * (upper - lower)


cdef inline uint rand_i(int upper) nogil:
    """A random integer between 0 <= r < upper."""
    return rand() % upper


@cython.final
@cython.freelist(100)
cdef class Chromosome:

    """Data structure class."""

    def __cinit__(self, uint n):
        self.f = 0.
        self.v = zeros(n, dtype=np_float)

    cdef void assign(self, Chromosome other):
        """Assign from an old generation."""
        if other is self:
            return
        self.f = other.f
        self.v[:] = other.v

    @staticmethod
    cdef Chromosome[:] new_pop(uint d, uint n):
        """Create new population."""
        return np_array([Chromosome.__new__(Chromosome, d) for _ in range(n)])


cdef class Objective:

    """Objective function class base."""

    cpdef double[:] get_upper(self):
        """Return upper bound."""
        raise NotImplementedError

    cpdef double[:] get_lower(self):
        """Return lower bound."""
        raise NotImplementedError

    cdef double fitness(self, double[:] v):
        """Calculate the fitness.

        Usage:
        f = MyObjective()
        fitness = f(chromosome.v)
        """
        raise NotImplementedError

    cpdef object result(self, double[:] v):
        """Show the result."""
        raise NotImplementedError


cdef class AlgorithmBase:

    """Algorithm base class."""

    def __cinit__(
        self,
        Objective func,
        dict settings,
        object progress_fun=None,
        object interrupt_fun=None
    ):
        """Generic settings."""
        srand(time(NULL))
        # object function
        self.func = func
        self.stop_at_i = 0
        self.stop_at_f = 0.
        if 'max_gen' in settings:
            self.stop_at = MAX_GEN
            self.stop_at_i = settings['max_gen']
        elif 'min_fit' in settings:
            self.stop_at = MIN_FIT
            self.stop_at_f = settings['min_fit']
        elif 'max_time' in settings:
            self.stop_at = MAX_TIME
            self.stop_at_f = settings['max_time']
        elif 'slow_down' in settings:
            self.stop_at = SLOW_DOWN
            self.stop_at_f = 1 - settings['slow_down']
        else:
            raise ValueError("please give 'max_gen', 'min_fit' or 'max_time' limit")
        self.rpt = settings.get('report', 0)
        if self.rpt <= 0:
            self.rpt = 10
        self.progress_fun = progress_fun
        self.interrupt_fun = interrupt_fun
        self.lb = self.func.get_lower()
        self.ub = self.func.get_upper()
        self.dim = len(self.ub)
        if self.dim != len(self.lb):
            raise ValueError("length of upper and lower bounds must be equal")
        self.last_best = Chromosome.__new__(Chromosome, self.dim)
        # setup benchmark
        self.gen = 0
        self.time_start = 0
        self.fitness_time = []

    cdef void initialize(self):
        """Initialize function."""
        raise NotImplementedError

    cdef void generation_process(self):
        """The process of each generation."""
        raise NotImplementedError

    cdef inline void report(self):
        """Report generation, fitness and time."""
        self.fitness_time.append((self.gen, self.last_best.f, process_time() - self.time_start))

    cpdef list history(self):
        """Return the history of the process."""
        return self.fitness_time

    cpdef object run(self):
        """Init and run GA for max_gen times."""
        self.time_start = process_time()
        self.initialize()
        self.report()

        cdef double diff, last_best
        cdef double last_diff = 0
        while True:
            last_best = self.last_best.f
            self.gen += 1
            self.generation_process()
            if self.gen % self.rpt == 0:
                self.report()
            if self.stop_at == MAX_GEN:
                if self.gen >= self.stop_at_i > 0:
                    break
            elif self.stop_at == MIN_FIT:
                if self.last_best.f <= self.stop_at_f:
                    break
            elif self.stop_at == MAX_TIME:
                if process_time() - self.time_start >= self.stop_at_f > 0:
                    break
            elif self.stop_at == SLOW_DOWN:
                diff = last_best - self.last_best.f
                if last_diff > 0 and diff / last_diff >= self.stop_at_f:
                    break
                last_diff = diff
            # progress
            if self.progress_fun is not None:
                self.progress_fun(self.gen, f"{self.last_best.f:.04f}")
            # interrupt
            if (self.interrupt_fun is not None) and self.interrupt_fun():
                break
        self.report()
        return self.func.result(self.last_best.v)



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Differential Evolution

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from .utility cimport (
    rand_v,
    rand_i,
    Chromosome,
    Objective,
    AlgorithmBase,
)

ctypedef unsigned int uint
ctypedef void (*Eq)(Differential, int, Chromosome)


cdef enum Strategy:
    STRATEGY0
    STRATEGY1
    STRATEGY2
    STRATEGY3
    STRATEGY4
    STRATEGY5
    STRATEGY6
    STRATEGY7
    STRATEGY8
    STRATEGY9


@cython.final
cdef class Differential(AlgorithmBase):

    """Algorithm class."""

    cdef Strategy strategy
    cdef uint np, r1, r2, r3, r4, r5
    cdef double F, CR
    cdef Chromosome[:] pool

    def __cinit__(
        self,
        Objective func,
        dict settings,
        object progress_fun=None,
        object interrupt_fun=None
    ):
        """
        settings = {
            'strategy': int,
            'NP': int,
            'F': float,
            'CR': float,
            'max_gen': int or 'min_fit': float or 'max_time': float,
            'report': int,
        }
        """
        # strategy 0~9, choice what strategy to generate new member in temporary
        cdef uint strategy = settings.get('strategy', STRATEGY1)
        if strategy > 9:
            raise ValueError(f"invalid strategy: {strategy}")
        self.strategy = <Strategy>strategy
        # population size
        # To start off np = 10*dim is a reasonable choice. Increase np if misconvergence
        self.np = settings.get('NP', 400)
        # weight factor F is usually between 0.5 and 1 (in rare cases > 1)
        self.F = settings.get('F', 0.6)
        if not (0.5 <= self.F <= 1):
            raise ValueError('CR should be [0.5,1]')
        # crossover possible CR in [0,1]
        self.CR = settings.get('CR', 0.9)
        if not (0 <= self.CR <= 1):
            raise ValueError('CR should be [0,1]')
        # the vector
        self.r1 = self.r2 = self.r3 = self.r4 = self.r5 = 0
        # generation pool, depended on population size
        self.pool = Chromosome.new_pop(self.dim, self.np)

    cdef inline void initialize(self):
        """Initial population."""
        cdef uint i, j
        cdef Chromosome tmp
        for i in range(self.np):
            tmp = self.pool[i]
            for j in range(self.dim):
                tmp.v[j] = rand_v(self.lb[j], self.ub[j])
            tmp.f = self.func.fitness(tmp.v)
        self.last_best.assign(self.find_best())

    cdef inline Chromosome find_best(self):
        """Find member that have minimum fitness value from pool."""
        cdef Chromosome best = self.pool[0]
        cdef Chromosome tmp
        for tmp in self.pool[1:]:
            if tmp.f < best.f:
                best = tmp
        return best

    cdef inline void generate_random_vector(self, uint i):
        """Generate new vector."""
        self.r1 = self.r2 = self.r3 = self.r4 = self.r5 = i
        while self.r1 == i:
            self.r1 = rand_i(self.np)
        while self.r2 in {i, self.r1}:
            self.r2 = rand_i(self.np)
        if self.strategy in {STRATEGY1, STRATEGY3, STRATEGY6, STRATEGY8}:
            return
        while self.r3 in {i, self.r1, self.r2}:
            self.r3 = rand_i(self.np)
        if self.strategy in {STRATEGY2, STRATEGY7}:
            return
        while self.r4 in {i, self.r1, self.r2, self.r3}:
            self.r4 = rand_i(self.np)
        if self.strategy in {STRATEGY4, STRATEGY9}:
            return
        while self.r5 in {i, self.r1, self.r2, self.r3, self.r4}:
            self.r5 = rand_i(self.np)

    cdef inline void type1(self, Chromosome tmp, Eq func):
        cdef uint n = rand_i(self.dim)
        cdef uint l_v = 0
        while True:
            func(self, n, tmp)
            n = (n + 1) % self.dim
            l_v += 1
            if not (rand_v() < self.CR and l_v < self.dim):
                break

    cdef inline void type2(self, Chromosome tmp, Eq func):
        cdef uint n = rand_i(self.dim)
        cdef uint l_v
        for l_v in range(self.dim):
            if rand_v() < self.CR or l_v == self.dim - 1:
                func(self, n, tmp)
            n = (n + 1) % self.dim

    cdef void eq1(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        tmp.v[n] = self.last_best.v[n] + self.F * (c1.v[n] - c2.v[n])

    cdef void eq2(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        cdef Chromosome c3 = self.pool[self.r3]
        tmp.v[n] = c1.v[n] + self.F * (c2.v[n] - c3.v[n])

    cdef void eq3(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        tmp.v[n] = tmp.v[n] + self.F * (self.last_best.v[n] - tmp.v[n]) + self.F * (c1.v[n] - c2.v[n])

    cdef void eq4(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        cdef Chromosome c3 = self.pool[self.r3]
        cdef Chromosome c4 = self.pool[self.r4]
        tmp.v[n] = self.last_best.v[n] + (c1.v[n] + c2.v[n] - c3.v[n] - c4.v[n]) * self.F

    cdef void eq5(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        cdef Chromosome c3 = self.pool[self.r3]
        cdef Chromosome c4 = self.pool[self.r4]
        cdef Chromosome c5 = self.pool[self.r5]
        tmp.v[n] = c5.v[n] + (c1.v[n] + c2.v[n] - c3.v[n] - c4.v[n]) * self.F

    cdef inline Chromosome recombination(self, int i):
        """use new vector, recombination the new one member to tmp."""
        cdef Chromosome tmp = Chromosome.__new__(Chromosome, self.dim)
        tmp.assign(self.pool[i])
        if self.strategy == 1:
            self.type1(tmp, Differential.eq1)
        elif self.strategy == 2:
            self.type1(tmp, Differential.eq2)
        elif self.strategy == 3:
            self.type1(tmp, Differential.eq3)
        elif self.strategy == 4:
            self.type1(tmp, Differential.eq4)
        elif self.strategy == 5:
            self.type1(tmp, Differential.eq5)
        elif self.strategy == 6:
            self.type2(tmp, Differential.eq1)
        elif self.strategy == 7:
            self.type2(tmp, Differential.eq2)
        elif self.strategy == 8:
            self.type2(tmp, Differential.eq3)
        elif self.strategy == 9:
            self.type2(tmp, Differential.eq4)
        elif self.strategy == 0:
            self.type2(tmp, Differential.eq5)
        return tmp

    cdef inline bint over_bound(self, Chromosome member):
        """check the member's chromosome that is out of bound?"""
        cdef uint i
        for i in range(self.dim):
            if member.v[i] > self.ub[i] or member.v[i] < self.lb[i]:
                return True
        return False

    cdef inline void generation_process(self):
        cdef uint i
        cdef Chromosome tmp, baby
        for i in range(self.np):
            # generate new vector
            self.generate_random_vector(i)
            # use the vector recombine the member to temporary
            tmp = self.recombination(i)
            # check the one is out of bound?
            if self.over_bound(tmp):
                # if it is, then abandon it
                continue
            # is not out of bound, that mean it's qualify of environment
            # then evaluate the one
            tmp.f = self.func.fitness(tmp.v)
            # if temporary one is better than origin(fitness value is smaller)
            baby = self.pool[i]
            if tmp.f <= baby.f:
                # copy the temporary one to origin member
                baby.assign(tmp)
                # check the temporary one is better than the current_best
                if tmp.f < self.last_best.f:
                    # copy the temporary one to current_best
                    self.last_best.assign(tmp)



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Real-coded Genetic Algorithm

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from libc.math cimport pow, HUGE_VAL
from .utility cimport (
    MAX_GEN,
    rand_v,
    rand_i,
    Chromosome,
    Objective,
    AlgorithmBase,
)

ctypedef unsigned int uint


@cython.final
cdef class Genetic(AlgorithmBase):

    """Algorithm class."""

    cdef uint pop_num
    cdef double cross, mute, win, delta
    cdef Chromosome[:] chromosome, new_chromosome

    def __cinit__(
        self,
        Objective func,
        dict settings,
        object progress_fun=None,
        object interrupt_fun=None
    ):
        """
        settings = {
            'pop_num': int,
            'cross': float,
            'mute': float,
            'win': float,
            'delta': float,
            'max_gen': int or 'min_fit': float or 'max_time': float,
            'report': int,
        }
        """
        self.pop_num = settings.get('pop_num', 500)
        self.cross = settings.get('cross', 0.95)
        self.mute = settings.get('mute', 0.05)
        self.win = settings.get('win', 0.95)
        self.delta = settings.get('delta', 5.)
        self.chromosome = Chromosome.new_pop(self.dim, self.pop_num)
        self.new_chromosome = Chromosome.new_pop(self.dim, self.pop_num)

    cdef inline double check(self, int i, double v):
        """If a variable is out of bound, replace it with a random value."""
        if v > self.ub[i] or v < self.lb[i]:
            return rand_v(self.lb[i], self.ub[i])
        return v

    cdef inline void initialize(self):
        cdef uint i, j
        cdef Chromosome tmp
        for i in range(self.pop_num):
            tmp = self.chromosome[i]
            for j in range(self.dim):
                tmp.v[j] = rand_v(self.lb[j], self.ub[j])
        tmp = self.chromosome[0]
        tmp.f = self.func.fitness(tmp.v)
        self.last_best.assign(tmp)
        self.fitness()

    cdef inline void cross_over(self):
        cdef Chromosome c1 = Chromosome.__new__(Chromosome, self.dim)
        cdef Chromosome c2 = Chromosome.__new__(Chromosome, self.dim)
        cdef Chromosome c3 = Chromosome.__new__(Chromosome, self.dim)
        cdef uint i, s
        cdef Chromosome b1, b2
        for i in range(0, <uint>(self.pop_num - 1), 2):
            if not rand_v() < self.cross:
                continue

            b1 = self.chromosome[i]
            b2 = self.chromosome[i + 1]
            for s in range(self.dim):
                # first baby, half father half mother
                c1.v[s] = 0.5 * b1.v[s] + 0.5 * b2.v[s]
                # second baby, three quarters of father and quarter of mother
                c2.v[s] = self.check(s, 1.5 * b1.v[s] - 0.5 * b2.v[s])
                # third baby, quarter of father and three quarters of mother
                c3.v[s] = self.check(s, -0.5 * b1.v[s] + 1.5 * b2.v[s])
            # evaluate new baby
            c1.f = self.func.fitness(c1.v)
            c2.f = self.func.fitness(c2.v)
            c3.f = self.func.fitness(c3.v)
            # bubble sort: smaller -> larger
            if c1.f > c2.f:
                c1, c2 = c2, c1
            if c1.f > c3.f:
                c1, c3 = c3, c1
            if c2.f > c3.f:
                c2, c3 = c3, c2
            # replace first two baby to parent, another one will be
            b1.assign(c1)
            b2.assign(c2)

    cdef inline double get_delta(self, double y):
        cdef double r
        if self.stop_at == MAX_GEN and self.stop_at_i > 0:
            r = <double>self.gen / self.stop_at_i
        else:
            r = 1
        return y * rand_v() * pow(1.0 - r, self.delta)

    cdef inline void fitness(self):
        cdef uint i
        cdef Chromosome tmp
        for i in range(self.pop_num):
            tmp = self.chromosome[i]
            tmp.f = self.func.fitness(tmp.v)

        cdef int index = 0
        cdef double f = HUGE_VAL

        for i, tmp in enumerate(self.chromosome):
            if tmp.f < f:
                index = i
                f = tmp.f
        if f < self.last_best.f:
            self.last_best.assign(self.chromosome[index])

    cdef inline void mutate(self):
        cdef uint i, s
        cdef Chromosome tmp
        for i in range(self.pop_num):
            if not rand_v() < self.mute:
                continue
            s = rand_i(self.dim)
            tmp = self.chromosome[i]
            if rand_v() < 0.5:
                tmp.v[s] += self.get_delta(self.ub[s] - tmp.v[s])
            else:
                tmp.v[s] -= self.get_delta(tmp.v[s] - self.lb[s])

    cdef inline void select(self):
        """roulette wheel selection"""
        cdef uint i, j, k
        cdef Chromosome baby, b1, b2
        for i in range(self.pop_num):
            j = rand_i(self.pop_num)
            k = rand_i(self.pop_num)
            b1 = self.chromosome[j]
            b2 = self.chromosome[k]
            baby = self.new_chromosome[i]
            if b1.f > b2.f and rand_v() < self.win:
                baby.assign(b2)
            else:
                baby.assign(b1)
        # in this stage, new_chromosome is select finish
        # now replace origin chromosome
        for i in range(self.pop_num):
            baby = self.chromosome[i]
            baby.assign(self.new_chromosome[i])
        # select random one chromosome to be best chromosome, make best chromosome still exist
        baby = self.chromosome[rand_i(self.pop_num)]
        baby.assign(self.last_best)

    cdef inline void generation_process(self):
        self.select()
        self.cross_over()
        self.mutate()
        self.fitness()



# -*- coding: utf-8 -*-

from typing import Tuple, List, Sequence, FrozenSet, Dict, Optional, Union
from .expression import VPoint, Coordinate

_Coord = Tuple[float, float]
_Inputs = Dict[Tuple[int, int], float]

class SolverSystem:

    """Sketch Solve solver."""

    def __init__(
        self,
        vpoints: Sequence[VPoint],
        inputs: Optional[Dict[Tuple[int, int], float]] = None,
        data_dict: Optional[Dict[Union[int, Tuple[int, int]], Union[Coordinate, float]]] = None
    ) -> None:
        ...

    def same_points(self, vpoints_: Sequence[VPoint]) -> bool:
        """Return True if two expressions are same."""
        ...

    def show_inputs(self) -> FrozenSet[Tuple[int, int]]:
        """Show the current inputs keys."""
        ...

    def show_data(self) -> FrozenSet[Union[int, Tuple[int, int]]]:
        """Show the current data keys."""
        ...

    def set_inputs(self, inputs: Dict[Tuple[int, int], float]) -> None:
        """Set input pairs."""
        ...

    def set_data(self, data_dict: Union[_Inputs, Dict[int, Coordinate]]) -> None:
        """Set data."""
        ...

    def solve(self) -> List[Union[_Coord, Tuple[_Coord, _Coord]]]:
        """Solve the expression."""
        ...



# -*- coding: utf-8 -*-
# cython: language_level=3

"""Sharing position analysis function.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from .triangulation cimport ExpressionStack
from .expression cimport Coordinate

cdef double radians(double degree) nogil
cpdef Coordinate plap(Coordinate c1, double d0, double a0, Coordinate c2=*, bint inverse=*)
cpdef Coordinate pllp(Coordinate c1, double d0, double d1, Coordinate c2, bint inverse=*)
cpdef Coordinate plpp(Coordinate c1, double d0, Coordinate c2, Coordinate c3, bint inverse=*)
cpdef Coordinate pxy(Coordinate c1, double x, double y)

cdef str str_between(str s, str front, str back)
cdef str str_before(str s, str front)

cpdef void expr_parser(ExpressionStack exprs, dict data_dict)
cpdef tuple data_collecting(ExpressionStack exprs, dict mapping, object vpoints_)
cpdef list expr_solving(ExpressionStack exprs, dict mapping, object vpoints, object angles=*)



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Teaching Learning Based Optimization

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from numpy cimport ndarray
from numpy import zeros, float64 as np_float
from .utility cimport (
    rand_v,
    rand_i,
    Chromosome,
    Objective,
    AlgorithmBase,
)

ctypedef unsigned int uint


@cython.final
cdef class TeachingLearning(AlgorithmBase):

    """Algorithm class."""

    cdef uint class_size
    cdef Chromosome[:] students

    def __cinit__(
        self,
        Objective func,
        dict settings,
        object progress_fun=None,
        object interrupt_fun=None
    ):
        """
        settings = {
            'class_size': int,
            'max_gen': int or 'min_fit': float or 'max_time': float,
            'report': int,
        }
        """
        self.class_size = settings.get('class_size', 50)
        self.students = Chromosome.new_pop(self.dim, self.class_size)

    cdef inline void initialize(self):
        """Initial population: Sorted students."""
        cdef ndarray[double, ndim=2] s = zeros((self.class_size, self.dim + 1), dtype=np_float)
        cdef uint i, j
        for i in range(self.class_size):
            for j in range(self.dim):
                s[i, j] = rand_v(self.lb[j], self.ub[j])
            s[i, -1] = self.func.fitness(s[i, :-1])
        s = s[s[:, -1].argsort()][::-1]
        for i in range(self.class_size):
            self.students[i].v = s[i, :-1]
            self.students[i].f = s[i, -1]
        self.last_best.assign(self.students[-1])

    cdef inline void teaching(self, uint index):
        """Teaching phase. The last best is the teacher."""
        cdef Chromosome student = self.students[index]
        cdef double[:] v = zeros(self.dim, dtype=np_float)
        cdef double tf = round(1 + rand_v())
        cdef uint i, j
        cdef double mean
        cdef Chromosome tmp
        for i in range(self.dim):
            if self.state_check():
                return
            mean = 0
            for j in range(self.class_size):
                mean += self.students[j].v[i]
            mean /= self.dim
            v[i] = student.v[i] + rand_v(1, self.dim) * (self.last_best.v[i] - tf * mean)
            if v[i] < self.lb[i]:
                v[i] = self.lb[i]
            elif v[i] > self.ub[i]:
                v[i] = self.ub[i]
        cdef double f_new = self.func.fitness(v)
        if f_new < student.f:
            student.v[:] = v
            student.f = f_new
        if student.f < self.last_best.f:
            self.last_best.assign(student)

    cdef inline void learning(self, uint index):
        """Learning phase."""
        cdef Chromosome student_a = self.students[index]
        cdef uint cmp_index = rand_i(self.class_size - 1)
        if cmp_index >= index:
            cmp_index += 1
        cdef Chromosome student_b = self.students[cmp_index]
        cdef double[:] v = zeros(self.dim, dtype=np_float)
        cdef uint i
        cdef double diff
        for i in range(self.dim):
            if self.state_check():
                return
            if student_b.f < student_a.f:
                diff = student_a.v[i] - student_b.v[i]
            else:
                diff = student_b.v[i] - student_a.v[i]
            v[i] = student_a.v[i] + diff * rand_v(1, self.dim)
            if v[i] < self.lb[i]:
                v[i] = self.lb[i]
            elif v[i] > self.ub[i]:
                v[i] = self.ub[i]
        cdef double f_new = self.func.fitness(v)
        if f_new < student_a.f:
            student_a.v[:] = v
            student_a.f = f_new
        if student_a.f < self.last_best.f:
            self.last_best.assign(student_a)

    cdef inline bint state_check(self):
        """Check status."""
        if self.progress_fun is not None:
            self.progress_fun(self.gen, f"{self.last_best.f:.04f}")
        if (self.interrupt_fun is not None) and self.interrupt_fun():
            return True
        return False

    cdef inline void generation_process(self):
        """The process of each generation."""
        cdef uint i
        for i in range(self.class_size):
            if self.state_check():
                break
            self.teaching(i)
            self.learning(i)



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""PMKS symbolics.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from libc.math cimport M_PI, atan2, hypot
from cpython.object cimport Py_EQ, Py_NE
from typing import Iterable
from numpy import array as np_array


cdef inline double distance(double x1, double y1, double x2, double y2) nogil:
    """Distance of two cartesian coordinates."""
    return hypot(x2 - x1, y2 - y1)


cpdef list get_vlinks(object vpoints):
    """Get VLinks of a list of VPoint."""
    links = {}
    cdef int i
    cdef VPoint vpoint
    for i, vpoint in enumerate(vpoints):
        for name in vpoint.links:
            if name not in links:
                links[name] = {i}
            else:
                links[name].add(i)
    vlinks = []
    for name, points in links.items():
        vlinks.append(VLink(name, "", points))
    return vlinks


@cython.final
cdef class Coordinate:

    """A class to store the coordinate."""

    def __cinit__(self, double x, double y):
        self.x = x
        self.y = y

    cpdef double distance(self, Coordinate p):
        """Distance."""
        return distance(self.x, self.y, p.x, p.y)

    cpdef bint is_nan(self):
        """Test this coordinate is a error-occurred answer."""
        return self.x != self.x

    def __repr__(self):
        return f"Coordinate({self.x:.02f}, {self.y:.02f})"


@cython.final
cdef class VPoint:

    """Symbol of joints."""

    HOLDER = VPoint([], VJoint.R, 0., "", 0., 0.)

    def __cinit__(
        self,
        object links,
        VJoint j_type,
        double angle,
        str color_str,
        double x,
        double y,
        object color_func=None
    ):
        self.links = tuple(links)
        self.type = j_type
        self.type_str = ('R', 'P', 'RP')[j_type]
        self.angle = angle

        self.color_str = color_str
        if color_func is None:
            self.color = None
        else:
            self.color = color_func(color_str)

        self.x = x
        self.y = y
        self.c = ndarray(2, dtype=object)

        if self.type in {VJoint.P, VJoint.RP}:
            # Slider current coordinates.
            # [0]: Current node on slot.
            # [1]: Pin.
            self.c[0] = (self.x, self.y)
            self.c[1] = (self.x, self.y)
        else:
            self.c[0] = (self.x, self.y)

        self.__has_offset = False
        self.__offset = 0

    @staticmethod
    def r_joint(links: Iterable[str], x: double, y: double) -> VPoint:
        """Create by coordinate."""
        return VPoint.c_r_joint(links, x, y)

    @staticmethod
    cdef VPoint c_r_joint(object links, double x, double y):
        return VPoint.__new__(VPoint, links, VJoint.R, 0., '', x, y)

    @staticmethod
    def slider_joint(links: Iterable[str], type_int: VJoint, angle: double, x: double, y: double) -> VPoint:
        """Create by coordinate."""
        return VPoint.c_slider_joint(links, type_int, angle, x, y)

    @staticmethod
    cdef VPoint c_slider_joint(object links, VJoint type_int, double angle, double x, double y):
        return VPoint.__new__(VPoint, links, type_int, angle, '', x, y)

    cpdef VPoint copy(self):
        """Copy method of Python."""
        return self.__copy__()

    @property
    def cx(self) -> float:
        """X value of first current coordinate."""
        if self.type == VJoint.R:
            return self.c[0][0]
        else:
            return self.c[1][0]

    @property
    def cy(self) -> float:
        """Y value of first current coordinate."""
        if self.type == VJoint.R:
            return self.c[0][1]
        else:
            return self.c[1][1]

    cpdef void set_links(self, object links) except *:
        """Set links."""
        self.links = tuple(links)

    cpdef void replace_link(self, str link1, str link2) except *:
        """Replace link1 as link2."""
        self.links = tuple([link2 if link == link1 else link for link in self.links])

    cpdef void move(self, tuple c1, tuple c2 = None) except *:
        """Change the coordinates of this point."""
        cdef double x, y
        x, y = c1
        self.c[0] = (x, y)
        if self.type in {VJoint.P, VJoint.RP}:
            if c2 is not None:
                x, y = c2
            self.c[1] = (x, y)

    cpdef void locate(self, double x, double y) except *:
        """Change the origin coordinate of this point directly."""
        self.x = x
        self.y = y
        self.move((x, y))

    cpdef void rotate(self, double angle):
        """Change the angle of slider slot by degrees."""
        self.angle = angle % 180

    cpdef void set_offset(self, double offset):
        """Set slider offset."""
        self.__has_offset = True
        self.__offset = offset

    cpdef void disable_offset(self):
        """Disable offset status."""
        self.__has_offset = False

    cpdef double distance(self, VPoint p):
        """Distance between two VPoint."""
        on_links = tuple(set(self.links) & set(p.links))
        cdef double m_x = 0
        cdef double m_y = 0
        cdef double p_x = 0
        cdef double p_y = 0
        if on_links:
            if self.type == VJoint.R or self.links[0] == on_links[0]:
                # self is R joint or at base link.
                m_x = self.c[0][0]
                m_y = self.c[0][1]
            else:
                # At pin joint.
                m_x = self.c[1][0]
                m_y = self.c[1][1]
            if p.type == VJoint.R or p.links[0] == on_links[0]:
                # p is R joint or at base link.
                p_x = p.c[0][0]
                p_y = p.c[0][1]
            else:
                # At pin joint.
                p_x = p.c[1][0]
                p_y = p.c[1][1]
        else:
            m_x = self.c[0][0]
            m_y = self.c[0][1]
            p_x = p.c[0][0]
            p_y = p.c[0][1]
        return hypot(p_x - m_x, p_y - m_y)

    cpdef bint has_offset(self):
        """Return has offset."""
        return self.__has_offset

    cpdef double offset(self):
        """Return target offset."""
        return self.__offset

    cpdef double true_offset(self):
        """Return offset between slot and pin."""
        return hypot(self.c[1][0] - self.c[0][0], self.c[1][1] - self.c[0][1])

    cpdef double slope_angle(self, VPoint p, int num1 = 2, int num2 = 2):
        """Angle between horizontal line and two point.
        
        num1: me.
        num2: other side.
        [0]: base (slot) link.
        [1]: pin link.
        """
        cdef double x1, y1, x2, y2
        if num1 > 1:
            x2, y2 = self.x, self.y
        else:
            x2, y2 = self.c[num2]
        if num2 > 1:
            x1, y1 = p.x, p.y
        else:
            x1, y1 = p.c[num2]
        return atan2(y1 - y2, x1 - x2) / M_PI * 180

    cpdef bint grounded(self):
        """Return True if the joint is connect with the ground."""
        if self.type == VJoint.R:
            return VLink.FRAME in self.links
        elif self.type in {VJoint.P, VJoint.RP}:
            if self.links:
                return self.is_slot_link(VLink.FRAME)
            else:
                return False

    cpdef bint pin_grounded(self):
        """Return True if the joint has any pin connect with the ground."""
        return VLink.FRAME in self.links[1:]

    cpdef bint same_link(self, VPoint p):
        """Return True if the point is at the same link."""
        return set(self.links) & set(p.links)

    cpdef bint no_link(self):
        """Return True if the point has no link."""
        return not self.links

    cpdef bint is_slot_link(self, str link_name):
        """Return True if the link name is first link."""
        if self.type == VJoint.R:
            return False
        if self.links:
            return link_name == self.links[0]
        else:
            return False

    cpdef str expr(self):
        """Expression."""
        if self.type != VJoint.R:
            type_text = f"{self.type_str}, A[{self.angle}]"
        else:
            type_text = 'R'
        if self.color_str:
            color = f", color[{self.color_str}]"
        else:
            color = ""
        links_text = ", ".join([name for name in self.links])
        x_text = f"{self.x:.4f}".rstrip('0').rstrip('.')
        y_text = f"{self.y:.4f}".rstrip('0').rstrip('.')
        return f"J[{type_text}{color}, P[{x_text}, {y_text}], L[{links_text}]]"

    def __copy__(self) -> VPoint:
        """Copy method."""
        cdef VPoint vpoint = VPoint.__new__(
            VPoint,
            self.links,
            self.type,
            self.angle,
            self.color_str,
            self.x,
            self.y
        )
        vpoint.move(self.c[0], self.c[1])
        return vpoint

    def __richcmp__(self, VPoint other, int op) -> bint:
        """Rich comparison."""
        if op == Py_EQ:
            return (
                self.links == other.links and
                (self.c == other.c).all() and
                self.type == other.type and
                self.x == other.x and
                self.y == other.y and
                self.angle == other.angle
            )
        elif op == Py_NE:
            return (
                self.links != other.links or
                (self.c != other.c).any() or
                self.type != other.type or
                self.x != other.x or
                self.y != other.y or
                self.angle != other.angle
            )
        else:
            raise TypeError(
                f"'{op}' not support between instances of "
                f"{type(self)} and {type(other)}"
            )

    def __getitem__(self, i: int) -> float:
        """Get coordinate like this:

        x, y = VPoint(10, 20)
        """
        if self.type == VJoint.R:
            return self.c[0][i]
        else:
            return self.c[1][i]

    def __repr__(self) -> str:
        """Use to generate script."""
        return f"VPoint({self.links}, {int(self.type)}, {self.angle}, {list(self.c)})"


@cython.final
cdef class VLink:

    """Symbol of links."""

    HOLDER = VLink("", "", [])
    FRAME = 'ground'

    def __cinit__(
        self,
        str name,
        str color_str,
        object points,
        object color_func=None
    ):
        self.name = name
        self.color_str = color_str
        if color_func is None:
            self.color = None
        else:
            self.color = color_func(color_str)
        self.points = np_array(list(points), dtype=int)

    cpdef void set_points(self, object points) except *:
        """Set points."""
        self.points = np_array(list(points), dtype=int)

    def __contains__(self, point: int) -> bint:
        """Check if point number is in the link."""
        return point in self.points

    def __repr__(self) -> str:
        """Use to generate script."""
        return f"VLink('{self.name}', {tuple(self.points)}, color_qt)"



# -*- coding: utf-8 -*-

from typing import Tuple, List, Iterable, Dict
from numpy import ndarray

def link_assortment(g: Graph) -> List[int]:
    """Return link assortment of the graph."""
    ...

def contracted_link_assortment(g: Graph) -> List[int]:
    """Return contracted link assortment of the graph."""
    ...

def labeled_enumerate(g: Graph) -> List[Tuple[int, Graph]]:
    """Enumerate each node with labeled except isomorphism."""
    ...

class Graph:

    """NetworkX-like graph class."""

    edges: Tuple[Tuple[int, int], ...]
    vertices: Tuple[int, ...]

    def __init__(self, edges: Iterable[Tuple[int, int]]) -> None: ...

    def add_edge(self, n1: int, n2: int) -> None:
        """Add two vertices for an edge."""
        ...

    def add_vertices(self, vertices: Iterable[int]) -> None:
        """Add vertices from a tuple."""
        ...

    def dof(self) -> int:
        """Return degrees of freedom."""
        ...

    def neighbors(self, n: int) -> Tuple[int, ...]:
        """Neighbors except the node."""
        ...

    def degrees(self) -> Dict[int, int]:
        """Neighbors except the node."""
        ...

    def degree_code(self) -> int:
        """Return degree code of the graph."""
        ...

    def adjacency_matrix(self) -> ndarray:
        """Represent as adjacency matrix."""
        ...

    def is_connected(self, without: int = -1) -> bool:
        """Return True if the graph is not isolated."""
        ...

    def has_cut_link(self) -> bool:
        """Return True if the graph has any cut links."""
        ...

    def is_degenerate(self) -> bool:
        """Return True if this kinematic chain is degenerate.

        + Prue all multiple contracted links recursively.
        + Check the DOF of sub-graph if it is lower then zero.
        """
        ...

    def has_triangle(self) -> bool:
        """Return True if the graph has triangle."""
        ...

    def is_isomorphic(self, graph: Graph) -> bool:
        """Return True if two graphs is isomorphic."""
        ...

    def is_isomorphic_vf2(self, graph: Graph) -> bool:
        """Compare isomorphism by VF2 algorithm."""
        ...

    def is_isomorphic_degree_code(self, graph: Graph) -> bool:
        """Compare isomorphism by degree code algorithm."""
        ...

    def duplicate(self, vertices: Iterable[int], times: int) -> Graph:
        """Make graph duplicate by specific vertices. Return a new graph."""
        ...

    def copy(self) -> Graph:
        """Copy the graph."""
        ...



# -*- coding: utf-8 -*-

"""Compile the Cython libraries of Pyslvs."""

__author__ = "Yuan Chang"
__copyright__ = "Copyright (C) 2016-2019"
__license__ = "AGPL"
__email__ = "pyslvs@gmail.com"

import re
from sys import argv
from os import listdir
from os.path import sep, join as pth_join
from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext
from platform import system


def read(*path):
    with open(pth_join(*path), 'r', encoding='utf-8') as f:
        return f.read()


def find_version(*path):
    m = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", read(*path), re.M)
    if m:
        return m.group(1)
    raise RuntimeError("Unable to find version string.")


src_path = 'pyslvs'
bfgs_path = pth_join(src_path, 'bfgs_solver')
metaheuristics_path = pth_join(src_path, 'metaheuristics')
macros = [('_USE_MATH_DEFINES', None), ('M_PI', 'PI')]
compile_args = ['-O3', '-Wno-cpp', '-std=c++17']
if system() == 'Windows':
    # Disable format warning
    compile_args.append('-Wno-format')
# Disable NumPy warning
macros.append(('NPY_NO_DEPRECATED_API', 'NPY_1_7_API_VERSION'))

ext_modules = [Extension(
    src_path.replace(sep, '.') + '.bfgs',
    [
        pth_join(src_path, 'bfgs.pyx'),
        pth_join(bfgs_path, 'geometric_constraint.cpp'),
        pth_join(bfgs_path, 'derivatives.cpp'),
        pth_join(bfgs_path, 'solve.cpp'),
        pth_join(bfgs_path, 'calc.cpp'),
    ],
    language="c++",
    include_dirs=[bfgs_path]
)]
paths = [src_path, metaheuristics_path]
if 'test' in argv:
    paths.append('tests')
for place in paths:
    for source in listdir(place):
        if not source.endswith('.pyx'):
            continue
        if place == src_path and source == 'bfgs.pyx':
            continue
        ext_modules.append(Extension(
            place.replace(sep, '.') + '.' + source.split('.')[0],  # Base name
            [pth_join(place, source)],
            language="c++"
        ))


class Build(build_ext):
    def build_extensions(self):
        compiler = self.compiler.compiler_type
        if compiler in {'mingw32', 'unix'}:
            for e in self.extensions:
                e.define_macros = macros
                e.extra_compile_args = compile_args
        elif compiler == 'msvc':
            for e in self.extensions:
                e.define_macros = macros[:1]
                e.extra_compile_args = ['/O2']
        super(Build, self).build_extensions()

    def finalize_options(self):
        super(Build, self).finalize_options()
        import numpy
        self.include_dirs.append(numpy.get_include())


setup(
    name='pyslvs',
    version=find_version('pyslvs', '__init__.py'),
    author=__author__,
    author_email=__email__,
    license=__license__,
    description="Pyslvs library",
    long_description=read("README.md"),
    long_description_content_type='text/markdown',
    url="https://github.com/KmolYuan/pyslvs",
    packages=find_packages(exclude=('tests',)),
    package_data={'': ['*.pyi', '*.pxd', '*.pxy'], 'pyslvs': ['py.typed']},
    ext_modules=ext_modules,
    cmdclass={'build_ext': Build},
    zip_safe=False,
    python_requires=">=3.7",
    install_requires=read('requirements.txt').splitlines(),
    test_suite="tests",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Cython",
        "Topic :: Scientific/Engineering",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: OS Independent",
    ]
)



# -*- coding: utf-8 -*-
# cython: language_level=3

"""Number synthesis.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from numpy cimport int16_t

cdef int16_t[:, :] product(tuple pool, object stop_func)



# -*- coding: utf-8 -*-
# cython: language_level=3

"""Graph class.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from libcpp.map cimport map as cmap
from numpy cimport ndarray

ctypedef unsigned long long ullong
ctypedef cmap[int, int] imap

cpdef list link_assortment(Graph g)
cpdef list contracted_link_assortment(Graph g)

cdef class Graph:

    # Graph(edges)

    cdef readonly tuple edges
    cdef readonly tuple vertices
    cdef dict adj

    cpdef void add_vertices(self, object new_nodes)
    cpdef void add_edge(self, int n1, int n2)
    cpdef void add_path(self, object new_nodes)
    cpdef void remove_edge(self, int n1, int n2)

    cpdef int dof(self)
    cpdef tuple neighbors(self, int n)
    cpdef dict degrees(self)
    cpdef ullong degree_code(self)
    cpdef ndarray adjacency_matrix(self)

    cpdef bint is_connected(self, int without=*)
    cpdef bint has_cut_link(self)
    cpdef bint is_degenerate(self)
    cpdef bint has_triangle(self)
    cpdef bint is_isomorphic(self, Graph g)
    cpdef bint is_isomorphic_vf2(self, Graph g)
    cpdef bint is_isomorphic_degree_code(self, Graph g)

    cpdef Graph duplicate(self, object vertices, int times)
    cpdef Graph copy(self)
